$(function () {
	/**
	 * Вывод карты в div под полем
	 */
	$(".show-map").on('click', function(e){
		e.preventDefault();
		var obj = $(this);
		var map_id = obj.attr('href');

		if(!$(this).hasClass('disable')) {
			$(map_id).parent('div').show();
			obj.text('Скрыть карту');
			obj.addClass('disable');
			$(document).trigger('show_map');
		} else {
			$(map_id).parent('div').hide();
			obj.text('Показать карту');
			obj.removeClass('disable');
		}
	});
});

/**
 * Инициализация Яндекс.Карт
 */
function initMapsYandex() {
	TAO.fields = TAO.fields || {};
	TAO.fields.map = TAO.fields.map || [];
	TAO.fields.map.push(initMaps('yandex'));
}

/**
 * Инициализация карт Google
 */
function initMapsGoogle() {
	TAO.fields = TAO.fields || {};
	TAO.fields.map = TAO.fields.map || [];
	TAO.fields.map.push(initMaps('google'));
}

/**
 * Общая функция иницилизации карт
 * @param service
 * @returns {MapBuilder}
 */
function initMaps(service) {
	var mapBuilder = new MapBuilder();
	mapBuilder.setSettings(TAO.settings.map);
	mapBuilder.buildMaps(service);
	return mapBuilder;
}

/**
 * Конструктор класса для построения карт
 * @constructor
 */
function MapBuilder() {
	this.settings = {};
	this.maps = {};
}

/**
 * Метод устанавливающий настройки для карт из переданных данных
 * Объект data имеет вид
 * {
 *  key: {
 *      field_name: "value",
 *      map_options: {
 *          markers: [
 *              0: {
 *                  latitude: (float)
 *                  longitude: (float)
 *                  zoom: (int)
 *              }
 *              ...
 *          ],
 *          type: "value",
 *          zoom: (int)
 *      },
 *      service: ('yandex'/'google')
 *  }
 *  ....
 * }
 *
 * @param data
 */
MapBuilder.prototype.setSettings = function (data) {
	this.settings = data;
};

/**
 * Метод инициализирует карты
 * @param service
 */
MapBuilder.prototype.buildMaps = function (service) {
	if (this.settings != 'undefined') {
		for (var i in this.settings) {
			if (service == this.settings[i].service) {
				this.maps[i] = this[this.settings[i].service](this.settings[i]);
			}
		}
	}
};

/**
 * инициализация яндекс карты
 * @param data
 * @returns {{mapCoordsFieldDom: MapObject, mapCoordsFieldMap: Yandex_map}}
 */
MapBuilder.prototype.yandex = function (data) {
	$(document).on('show_map', function () {
		var mapObject = new MapObject(data);
		var map = new Yandex_map(data, mapObject);

		return {
			mapCoordsFieldDom: mapObject,
			mapCoordsFieldMap: map
		};
	});
};

/**
 * инициализация карты гугла
 * @param data
 */
MapBuilder.prototype.google = function (data) {
	$(document).on('show_map', function () {

		var mapObject = new MapObject(data);
		var map = new Google_map(data, mapObject);

		return {
			mapCoordsFieldDom: mapObject,
			mapCoordsFieldMap: map
		};
	});
};

/**
 * Класс описывающий DOM объект дл работы с картой
 * @param map_settings
 * @constructor
 */
var MapObject = function (map_settings) {

	this.lat_input = $('#mainform_' + map_settings.field_name + '_latitude');
	this.lng_input = $('#mainform_' + map_settings.field_name + '_longitude');
	this.zoom_input = $('#mainform_' + map_settings.field_name + '_zoom');

	this.set_fields = function (coords, zoom) {
		this.lat_input.val(coords[0]);
		this.lng_input.val(coords[1]);
		this.zoom_input.val(zoom);
	};

	this.get_coords = function() {
		return {
			lat: parseFloat(this.lat_input.val().replace(',','.')),
			lng: parseFloat(this.lng_input.val().replace(',','.')),
			zoom: parseInt(this.zoom_input.val())
		};
	}
};



