<?php

Core::load('CMS.CLI');

class CMS_CLI_Stockroom extends CMS_CLI_Handler implements Core_ModuleInterface
{

	protected $manager = null;

	public function __construct()
	{
		parent::__construct();
		Core::load('CMS.Stockroom');
		$this->manager = CMS_Stockroom::manager();
	}

	protected function check_action($action)
	{
		if (!in_array($action, array('install', 'update', 'delete'))) {
			print("Unknow action [$action]\n");
			exit(1);
		}
	}

	public function tao_stockroom()
	{	
		$name = reset($this->args);
		if (is_null($name)) {
			$this->tao_update();
		} else {
			print("\nSearch component: $name\n");
			$list = $this->manager->components_list(array('search' => $name));
			if($list) {
				foreach ($list as $component) {
					if (strtolower($component->mnemocode) == strtolower($name)) {
						print("Find component $name\n");
						$component = $this->manager->load_item($component->id);
						if (!$component->installed()) {
							$action = isset($this->args[1]) ? trim($this->args[1]) : 'install';
							$this->check_action($action);
							print("Try to $action component\n");
							if ($action == 'install') {
								if (!$this->manager->validate_package_intall($component)) {
									print("Component dir exists\nRemove (Y/N)?");
									$s = trim(fgets(STDIN));
									if (strtolower($s) == 'y') {
										foreach ($component->not_install_if_exists as $entry) {
											IO_FS::rm($this->manager->validate_path($entry));
										}
									} else {
										$error = $this->manager->get_last_error();
									}
								}
								if (!$error && !$this->manager->install_package_files($component))
								{
									$error = $this->manager->get_last_error();
								}
							}
							if ($action == 'delete') {
								if (!$component->delete()) {
									$error = "Can't delete component " . $this->manager->get_last_error();
								}
							}
							// unknow action
						} else {
							$action = isset($this->args[1]) ? trim($this->args[1]) : 'update';
							$this->check_action($action);
							print("Try to $action component\n");
							if ($action == 'update') {
								if ($component->installed() == $component['version']) {
									print("Current version is actual\nUpdate anyway (Y/N)?");
									$s = trim(fgets(STDIN));
									if (strtolower($s) != 'y') {
										$error = "Current version is actual\n";
									}
								}
								if (!$error && ($result = $this->manager->update_package_files($component))) {
									if (!empty($result['updated'])) {
										print("Update files:\n");
									}
									foreach($result['updated'] as $k => $v) {
										print("$k -> $v\n");
									}
									if (!empty($result['not_updated'])) {
										print("Not updated files:\n");
									}
									foreach($result['not_updated'] as $k => $v) {
										print("$k -> $v\n");
									}

								}
							}
							if ($action == 'delete') {
								if (!$component->delete()) {
									$error = "Can't delete component " . $this->manager->get_last_error();
								}
							}
							if ($action == 'install') {
								$error = "Component already installed\n";
							}
							// unknow action
						}
						if (!$error) {
							$error = $this->manager->get_last_error();
						}
						if ($error) {
							print("Error: $error\n");
						} else {
							print("Success $action component $name\n");
							if ($action == 'install') {
								$coreload = trim($component['coreload']);
								print("Don't forget $coreload\n");
							}
						}
						return;
					}
				}
			} else {
				print("Component $name not found\n");
			}
		}
	}

	public function tao_s()
	{
		return call_user_func_array(array($this, 'tao_stockroom'), func_get_args());
	}

	public function tao_update()
	{
		$current_version = CMS::VERSION;
		if ($current_version == '0.0.0') {
			print("It seems that the current version of TAO is taken from git!!!\n");
		} else {
			print("Current TAO version: {$current_version}\n");
		}
		$info = $this->manager->get_core_version();
		if (is_string($info)) {
			print($info);
			return;
		}
		$remote_version = $info->version;
		print("Remote TAO version: {$remote_version}\n");
		if ($current_version == $remote_version) {
			print("Remote and current versions a equals!\n");
		}
		print("Update TAO from remote (Y/N)?\n");
		$s = trim(fgets(STDIN));
		if (strtolower($s) == 'y') {
			$this->manager->update_core();
		}
	}

	public function tao_update_composer()
	{
		$this->manager->update_composer();
	}
}