<?php
Core::load('Mail');

/**
 * Class CMS_ErrorsNotifier
 *
 * Класс отвечает за оповещения при ошибках на сайте. Через настройку errors_notifier можно управлять параметрами:
 * - enabled     Включает/выключает оповещения(true/false)
 * - error_types Указывается массив типов ошибок, которые нужно отслеживать([E_ERROR, E_PARSE...])
 * - recipients  Указываются адреса получателей оповещений - либо один адрес, либо массив
 * - subject     Тема письма
 * - from        Адрес отправителя
 *
 */
class CMS_ErrorsNotifier implements Core_ModuleInterface
{
	protected static $instance;

	public static function initialize()
	{
		$notifier = self::instance();
		if ($notifier->is_enabled()) {
			register_shutdown_function(array($notifier, 'on_shutdown'));
		}
	}

	public static function instance()
	{
		if (!static::$instance) {
			static::$instance = new self();
		}
		return static::$instance;
	}

	public function config()
	{
		return Config::all();
	}

	public function config_value($param_name)
	{
		if (!isset($this->config()->errors_notifier, $this->config()->errors_notifier->$param_name)) {
			return null;
		}
		return $this->config()->errors_notifier->$param_name;
	}

	public function is_enabled()
	{
		return $this->config_value('enabled');
	}

	public function on_shutdown()
	{
		$error = error_get_last();
		if (in_array($error['type'], $this->trackable_error_types())) {
			$this->notify($error);
		}
	}

	public function notify($error)
	{
		$mail = Mail::Message()
			->subject($this->subject())
			->from($this->from())
			->text($this->message_content($error));

		foreach ($this->recipients() as $recipient) {
			$mail->to($recipient);
			$this->transport()->send($mail);
		}
	}

	protected function trackable_error_types()
	{
		return $this->config_value('error_types') ?: array(E_ERROR, E_PARSE, E_COMPILE_ERROR, E_CORE_ERROR);
	}

	protected function subject()
	{
		return $this->config_value('subject') ?: 'Ошибка на сайте ' . CMS::site_host();
	}

	protected function from()
	{
		return $this->config_value('from') ?: 'noreply@' . CMS::site_host();
	}

	protected function recipients()
	{
		if ($config_recipients = $this->config_value('recipients')) {
			$recipients = is_array($config_recipients) ? $config_recipients : array($config_recipients);
		}
		return $recipients;
	}

	protected function message_content($error)
	{
		$server = $_SERVER;
		unset($server['PHP_AUTH_PW']);
		return date('d.m.Y - G:i:s')
			. "\n" . print_r($error, true)
			. "\n" . print_r($server, true)
			. "\n" . print_r($_COOKIE, true)
			. "\n" . print_r($_SESSION, true)
			. "\n" . print_r($_POST, true);
	}

	protected function transport()
	{
		return Mail_Transport::php();
	}
}