<?php

/**
 * @package CMS\ExceptionCatcher
 */
class CMS_ExceptionCatcher implements Core_ModuleInterface
{

	const MODULE = 'CMS.ExceptionCatcher';
	const VERSION = '0.0.0';

	/**
	 * @param Exception $exception
	 *
	 * @throws Exception
	 * @return void
	 */
	public static function run($exception)
	{
		if (property_exists(CMS::$cfg, 'debug') && property_exists(CMS::$cfg->debug, 'email') && $email = CMS::$cfg->debug->email) {
			self::send($exception, $email);
		}
		header("http/1.0 503 Service Temporarily Unavailable");
		if (property_exists(CMS::$cfg, 'debug') && property_exists(CMS::$cfg->debug, 'dump') && CMS::$cfg->debug->dump) {
			self::draw($exception);
			die;
		}
		throw($exception);
	}

	/**
	 * @param Exception $exception
	 * @param string    $email
	 *
	 * @return void
	 */
	public static function send($exception, $email)
	{
		$project = CMS::$cfg->debug->project ?: $_SERVER['SERVER_NAME'];
		$subj = "$project: exception";
		$headers = WS::adapter()->headers();
		$text = date('d.m.Y - G:i:s') . "\n";
		$text .= $_SERVER['REQUEST_METHOD'] . ' ' . $_SERVER['REQUEST_URI'] . "\n";
		$text .= $exception->getMessage() . "\n";
		$text .= 'FILE: ' . $exception->getFile() . "\n";
		$text .= 'LINE: ' . $exception->getLine() . "\n";
		$text .= "\n" .
			self::dump($_GET, 'GET') .
			self::dump($_POST, 'POST') .
			self::dump($_COOKIE, 'COOKIE') .
			self::dump($_SESSION, 'SESSION') .
			self::dump($_SERVER, 'SERVER') .
			self::dump($headers, 'HTTP REQUEST HEADERS');
		mail($email, $subj, $text);
	}

	/**
	 * @param array  $array
	 * @param string $title
	 *
	 * @return string
	 */
	public static function dump($array, $title)
	{
		unset($array['PHP_AUTH_PW']);
		unset($array['Authorization']);
		ob_start();
		$t = "-- $title -";
		while (strlen($t) < 50) {
			$t .= '-';
		}
		print "$t\n";
		if (sizeof($array) == 0) {
			print "empty\n";
		} else {
			foreach ($array as $k => $v) {
				print "$k: ";
				if (is_array($v) || is_object($v)) {
					print base64_encode(serialize($v));
				} else {
					print $v;
				}
				print "\n";
			}
		}
		print "\n";
		$rc = ob_get_contents();
		ob_end_clean();
		return $rc;
	}

	/**
	 * @param Exception $exception
	 *
	 * @return void
	 */
	public static function draw($exception)
	{
		$title = 'Exception';
		$theme = !empty(CMS::$cfg->debug->dump->theme) ? CMS::$cfg->debug->dump->theme : '';
		$opened = empty(CMS::$cfg->debug->dump->short_args);
		$display = !empty(CMS::$cfg->debug->dump->display) ? CMS::$cfg->debug->dump->display : 'xdebug';
		$display = (!$display || $display == 'default') ? 'exception' : 'exceptions/' . $display;
		$display .= '.phtml';
		include(CMS::views_path($display));
	}

	/**
	 * @param mixed $m
	 *
	 * @return string
	 */
	public static function render_array($m)
	{
		$out = "";
		$c = 0;
		foreach ($m as $key => $value) {
			if ($out != '') {
				$out .= ', ';
			}
			if ($key != $c) {
				$out .= "$key:";
			}
			$out .= self::render_value($value);
			$c++;
		}
		return $out;
	}

	/**
	 * @param mixed $value
	 *
	 * @todo gettype
	 * @return string
	 */
	public static function render_value($value)
	{
		if (is_string($value)) {
			return '"' . $value . '"';
		}
		if (is_numeric($value)) {
			return $value;
		}
		if ($value === true) {
			return 'true';
		}
		if ($value === false) {
			return 'false';
		}
		if (is_object($value)) {
			return get_class($value);
		}
		if (is_array($value)) {
			return '[' . self::render_array($value) . ']';
		}
		return '???';
	}

	/**
	 * @param string $path
	 *
	 * @return string
	 */
	public static function show_path($path)
	{
		if (CMS::$cfg->debug->dump->short_paths) {
			return self::substract_path($path, $_SERVER['DOCUMENT_ROOT']);
		}
		return $path;
	}

	/**
	 * @param string $path
	 * @param string $root
	 * @param int    $count
	 *
	 * @return string
	 */
	public static function substract_path($path, $root, $count = 0)
	{
		if ($pos = strrpos($path, '/tao/')) {
			return '../' . substr($path, $pos + 1);
		}
		$root = rtrim($root, '/');
		if (!$root) {
			return $path;
		}
		if (preg_match("~^$root~", $path)) {
			return ($count == 0 ? './' : str_repeat('../', $count)) . ltrim(str_replace($root, '', $path), '/');
		}
		$root_array = explode('/', $root);
		$shorten_root = implode('/', array_slice($root_array, 0, count($root_array) - 1));
		return self::substract_path($path, $shorten_root, $count + 1);
	}
}
