<?php
Core::load('CMS.Fields.ValueContainer');

/**
 * @package CMS\Fields
 */
class CMS_Fields implements Core_ModuleInterface
{

	const MODULE = 'CMS.Fields';
	const VERSION = '0.0.0';

	static $types = array();
	static $dirs = array();

	static function initialize($config)
	{
		//TODO: автоматически регистрировать тепы из CMS.Fields.Types.*
		CMS::field_type('input', 'CMS.Fields.Types.Input');
		CMS::field_type('float', 'CMS.Fields.Types.Float');
		CMS::field_type('hidden', 'CMS.Fields.Types.Hidden');
		CMS::field_type('radio', 'CMS.Fields.Types.Radio');
		CMS::field_type('checkbox', 'CMS.Fields.Types.Checkbox');
		CMS::field_type('select', 'CMS.Fields.Types.Select');
		CMS::field_type('textarea', 'CMS.Fields.Types.Textarea');
		CMS::field_type('sqldate', 'CMS.Fields.Types.SQLDate');
		CMS::field_type('datestr', 'CMS.Fields.Types.DateStr');
		CMS::field_type('sqldatestr', 'CMS.Fields.Types.SQLDateStr');
		CMS::field_type('upload', 'CMS.Fields.Types.Upload');
		CMS::field_type('ajaxupload', 'CMS.Fields.Types.AjaxUpload');
		CMS::field_type('subheader', 'CMS.Fields.Types.Subheader');
		CMS::field_type('email', 'CMS.Fields.Types.Email');
		CMS::field_type('attaches', 'CMS.Fields.Types.Attaches');
		CMS::field_type('image', 'CMS.Fields.Types.Image');
		CMS::field_type('protect', 'CMS.Fields.Types.Protect');
		CMS::field_type('html', 'CMS.Fields.Types.HTML');
		CMS::field_type('wiki', 'CMS.Fields.Types.Wiki');
		CMS::field_type('gallery', 'CMS.Fields.Types.Gallery');
		CMS::field_type('parms', 'CMS.Fields.Types.Parms');
		CMS::field_type('tree_select', 'CMS.Fields.Types.TreeSelect');
		CMS::field_type('multivalue', 'CMS.Fields.Types.Multivalue');
		CMS::field_type('multilink', 'CMS.Fields.Types.Multilink');
		CMS::field_type('static_multilink', 'CMS.Fields.Types.StaticMultilink');
		CMS::field_type('checkboxes', 'CMS.Fields.Types.Checkboxes');
		CMS::field_type('autocomplete', 'CMS.Fields.Types.Autocomplete');
		CMS::field_type('content', 'CMS.Fields.Types.Content');
		CMS::field_type('documents', 'CMS.Fields.Types.Documents');
		CMS::field_type('documents_grid', 'CMS.Fields.Types.DocumentsGrid');
		CMS::field_type('youtube', 'CMS.Fields.Types.Youtube');
		CMS::field_type('map_coords', 'CMS.Fields.Types.MapCoords');
		CMS::field_type('fieldset', 'CMS.Fields.Types.FieldSet');
		CMS::field_type('array', 'CMS.Fields.Types.Array');
		CMS::field_type('tags', 'CMS.Fields.Types.Tags');
		CMS::field_type('relations', 'CMS.Fields.Types.Relations');
		CMS::field_type('iframe', 'CMS.Fields.Types.IFrame');
		CMS::field_type('multilang_ext', 'CMS.Fields.Types.ExtendedMultilang');
	}

	static function type_by_class($class)
	{
		$class = Core_Types::virtual_class_name_for($class);
		return array_search($class, CMS::$fields_types);
	}

	static function validate_parms($parms)
	{
		if (is_string($parms)) {
			$parms = array('type' => $parms);
		}
		return $parms;
	}

	/**
	 * @param string|array $type
	 *
	 * @return CMS_Fields_AbstractField
	 *
	 * @throws Core_ModuleNotFoundException
	 * @throws Exception
	 */
	public static function type($type)
	{
		if (is_array($type)) {
			$type = isset($type['type']) ? $type['type'] : 'input';
		}
		if (!is_string($type) || $type == '') {
			$type = 'input';
		}
		$type = mb_strtolower($type);

		if (!isset(self::$types[$type])) {
			if (isset(CMS::$fields_types[$type])) {
				$module = CMS::$fields_types[$type];
			} else {
				if (isset(CMS::$fields_types['input'])) {
					$module = CMS::$fields_types['input'];
				} else {
					throw new Exception('Fields initialization error');
				}
			}

			$dir = IO_FS::File(Core::loader()->file_path_for($module))->dir_name;

			Core::autoload($module);

			$object = Core::make($module);
			$object->dir($dir);
			self::$types[$type] = $object;

		}
		return self::$types[$type];
	}

	static function assign_from_object($form, $object, $fields)
	{
		if (!$fields) {
			return false;
		}
		foreach ($fields as $name => $data) {
			$tobj = self::type($data);
			$tobj->assign_from_object($form, $object, $name, $data);
		}
	}

	static function assign_to_object($form, $object, $fields)
	{
		if (!$fields) {
			return false;
		}
		foreach ($fields as $name => $data) {
			$tobj = self::type($data);
			$tobj->assign_to_object($form, $object, $name, $data);
		}
	}

	static function form_fields($form, $fields, $item = false)
	{
		if (!$fields) {
			return $form;
		}
		foreach ($fields as $name => $data) {
			if ($item && is_object($item)) {
				$data['__item'] = $item;
				$data['__item_id'] = $item->id();
			}
			$tobj = self::type($data);
			$tobj->form_fields($form, $name, $data);
			$tobj->form_validator($form, $name, $data);
		}
		return $form;
	}

	static function process_form($form, $request)
	{
		$rc = array();
		$p = $form->process($request);
		if (!$p) {
			$rc = $form->validator->errors->property_errors;
			if(!count($rc)) {
				$rc = array('Неизвестная ошибка');
			}
		}
		return $rc;
	}

	static function form($form, $fields, $view, $layout = false)
	{
		if (!$fields) {
			return $view;
		}
		foreach ($fields as $name => $data) {
			$tobj = self::type($data);
			$tobj->view($view);
			$tobj->draw_in_layout($name, $data, $layout);
		}
	}

	static function process_schema($table, $fields, $engine = false)
	{
		Core::load('DB.Schema');
		$table_schema = self::fields_to_schema($fields, $table);
		if ($engine) {
			$table_schema['mysql_engine'] = $engine;
		}
		DB_Schema::process(array($table => $table_schema));
		foreach ($fields as $name => $data) {
			$type = self::type($data);
			$type->process_schema($name, $data, $table, $fields);
		}
	}

	static public function fields_to_columns($fields, $table_name = null, &$schema = array())
	{
		$schema = self::fields_to_schema($fields, $table_name, $schema);
		$res = array();
		$columns = isset($schema['columns']) ? $schema['columns'] : array();
		if (empty($columns) && !empty($table_name) && isset($schema[$table_name]) && isset($schema[$table_name]['columns'])) {
			$columns = $schema[$table_name]['columns'];
		}
		if (!empty($columns)) {
			foreach ($columns as $name => $data) {
				if (is_string($name)) {
					$res[] = $name;
				} else {
					if (isset($data['name'])) {
						$res[] = $data['name'];
					}
				}
			}
		}
		return $res;
	}

	private static function merge_schema($schema, $fschema, $table_name = null, $field_name = null)
	{
		// mark columns
		if ($field_name && isset($fschema['columns'])) {
			foreach ($fschema['columns'] as $idx => $column) {
				$fschema['columns'][$idx]['__from_field'] = $field_name;
			}
		}
		//extract data
		$default_table = array();
		$other = array();
		foreach ($fschema as $name => $value) {
			if (in_array($name, array('columns', 'indexes'))) {
				$default_table[$name] = $value;
			} else {
				$other[$name] = $value;
			}
		}
		// write data
		if (!empty($table_name) && isset($schema[$table_name])) {
			$schema[$table_name] = array_merge_recursive($schema[$table_name], $default_table);
			$schema = array_merge_recursive($schema, $other);
		} else {
			$schema = array_merge_recursive($schema, $default_table);
		}
		return $schema;
	}

	protected static $fields_to_schema_cache = array();

	static public function fields_to_schema($fields, $table_name = null, &$schema = array())
	{
		$serialize_keys = md5(serialize(array_keys($fields)));
		$ckey = $table_name ? $table_name . $serialize_keys : $serialize_keys;
		if (isset(self::$fields_to_schema_cache[$ckey])) {
			$schema = array_replace_recursive($schema, self::$fields_to_schema_cache[$ckey]);
			return $schema;
		}
		foreach ($fields as $field_name => $data) {
			$type = self::type($data);
			if (isset($data['sqltype']) && $data['sqltype'] === false) {
				continue;
			}
			$sqltype = isset($data['sqltype']) ? $data['sqltype'] : $type->sqltype();
			$fschema = $type->schema($field_name, $data);
			if (isset($data['sqltypes'])) {
				foreach ($data['sqltypes'] as $sub_name => $sub_type) {
					self::column_schema($sub_name, $sub_type, $schema, $table_name, $field_name);
				}
			} elseif ($sqltype) {
				self::column_schema($field_name, $sqltype, $schema, $table_name, $field_name);
			} elseif (!empty($fschema)) {
				$schema = self::merge_schema($schema, $fschema, $table_name, $field_name);
			} elseif ($sqltypes = $type->sqltypes($field_name, $data)) {
				foreach ($sqltypes as $sub_name => $sub_type) {
					self::column_schema($sub_name, $sub_type, $schema, $table_name, $field_name);
				}
			}
		}
		return self::$fields_to_schema_cache[$ckey] = $schema;
	}

	static function column_schema($db_name, $src, &$schema, $table_name = null, $field_name = null)
	{
		// In some cases, shit happens
		// if (isset($schema['columns'][$db_name])) {
		// 	throw new CMS_Fields_Exception("CMS.Fields: the same column names '{$db_name}' in table '{$table_name}'");
		// }
		$type = 'text';
		$length = false;
		$size = false;
		$index = false;
		$index_name = false;
		$index_type = false;
		$index_columns = array();
		$precision = false;
		$scale = false;
		$default = '';
		$not_default = false;
		$nn = true;
		$src = trim(strtolower($src));

		$parms = array();
		$unsetparms = array();

		// index
		if ($m = Core_Regexps::match_with_results('{(.+)(\s+index(_[a-z_\-0-9]+)?(\(.+\))?)}i', $src)) {
			$index = true;
			if (isset($m[3])) {
				$index_name = trim($m[3], ' _');
				if ($index_name === 'primary') {
					$index_type = 'primary key';
					$index_name = false;
				}
				if (Core_Strings::starts_with($index_name, 'unique')) {
					$index_type = 'unique';
					$index_name = trim(str_replace('unique', '', $index_name), '_ ');
				}
			}
			if (isset($m[4])) {
				$index_data = trim($m[4], '() ');
				if (!empty($index_data)) {
					foreach (explode(',', $index_data) as $ipart) {
						$ipart = trim($ipart);
						$int_val = filter_var($ipart, FILTER_VALIDATE_INT);
						if (Core_Strings::contains($ipart, ':')) {
							$index_columns[] = explode(':', $ipart);
						} else {
							if ($int_val !== false) {
								$index_columns[] = array($db_name, $ipart);
							} else {
								$index_columns[] = $ipart;
							}
						}
					}
				}
			}
			$src = trim(str_replace($m[2], '', $src));
		}

		// type
		$type = $src;
		if ($m = Core_Regexps::match_with_results('{^([a-z]+)(\((\d+)\))?}i', $src)) {
			$type = $m[1];
			if (isset($m[2])) {
				$length = trim($m[3]);
			} else {
				$length = '';
			}
		}

		// null
		if ($m = Core_Regexps::match_with_results('{\s+null}', $src)) {
			$nn = false;
		}

		// parms
		if ($m = Core_Regexps::match_with_results('{/(.*)$}', $src)) {
			$parms_str = trim($m[1]);
			foreach (explode(',', $parms_str) as $parm) {
				list($k, $v) = explode('=', $parm);
				$v = trim($v);
				$k = trim($k);
				$int_val = filter_var($v, FILTER_VALIDATE_INT);
				if ($int_val !== false) {
					$v = $int_val;
				} else {
					$bool_val = filter_var($v, FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE);
					if (!is_null($bool_val)) {
						$v = $bool_val;
					}
				}
				if ($v === 'null') {
					$v = null;
				}
				if ($v !== 'none') {
					$parms[$k] = $v;
				} else {
					$unsetparms[$k] = $v;
				}
			}
		}

		// process parms
		if ($type == 'bigtext' || $type == 'longtext') {
			$type = 'text';
			$size = 'big';
		}

		if ($type == 'tinyint') {
			$type = 'int';
			$size = 'tiny';
		}

		if ($type == 'price') {
			$type = 'numeric';
			$precision = 10;
			$scale = 2;
			$default = '0.00';
		}

		if ($type == 'datetime') {
			$default = '0000-00-00 00:00:00';
		}

		if ($type == 'date') {
			$default = '0000-00-00';
		}

		if ($type == 'int') {
			$default = 0;
		}

		if ($type == 'serial') {
			$default = null;
		}

		if ($type == 'float' || $type == 'double') {
			$type = 'float';
			$default = 0.0;
		}

		if ('timestamp' == $type) {
			$not_default = true;
		}

		// compose all in one

		$rc = array('name' => $db_name, 'type' => $type, 'not null' => $nn, 'index' => $index);
		if (!$not_default) {
			$rc['default'] = $default;
		}
		if ($length) {
			$rc['length'] = $length;
		}
		if ($size) {
			$rc['size'] = $size;
		}
		if ($precision) {
			$rc['precision'] = $precision;
		}
		if ($scale) {
			$rc['scale'] = $scale;
		}

		if ($idx = $rc['index']) {
			if (!$index_name) {
				$index_name = "idx_" . ($table_name ? str_replace('_', '', $table_name) : '') . "_{$db_name}";
			}
			if (empty($index_columns)) {
				$index_columns = array($db_name);
			}
			$idx = array('name' => $index_name, 'columns' => $index_columns);
			if ($index_type) {
				$idx['type'] = $index_type;
			}
			$schema['indexes'][] = $idx;
		}
		unset($rc['index']);

		foreach ($unsetparms as $k => $v) {
			unset($rc[$k]);
		}
		$rc = array_replace_recursive($rc, $parms);

		if ($type == 'serial') {
			$schema['indexes']['PRIMARY'] = array('name' => 'PRIMARY', 'type' => 'primary key', 'columns' => array($db_name));
		}

		$rc['__from_field'] = $field_name;
		if ($type !== 'empty') {
			$schema['columns'][$db_name] = $rc;
		}

		return $rc;
	}

}

//TODO: refactoring
abstract class CMS_Fields_AbstractField
{

	protected $dir;
	/**
	 * @var Templates_HTML_Template $view
	 */
	protected $view;
	private $temp_code = false;
	protected $dir_paths = array();
	protected static $tag_attrs = array('class', 'value', 'accept', 'style', 'align', 'href', 'width', 'height',
		'alt', 'title', 'autocomplete', 'autofocus', 'checked', 'disabled', 'form', 'formaction',
		'formenctype', 'formmethod', 'formnovalidate', 'formtarget', 'list', 'max', 'min', 'multiple',
		'name', 'pattern', 'placeholder', 'readonly', 'required', 'size', 'src', 'step'
	);

	public function __construct()
	{
	}

//CONFIGURE
//--------------------------------------------------
	/**
	 * Если возвращает true, то в контроллере администрирования табличных данных будет корректно обрабатываться загрузка файлов
	 *
	 * @return bool
	 */
	public function is_upload()
	{
		return false;
	}

	public function sqltype()
	{
		return false;
	}

	public function sqltypes($name, $data)
	{
		return false;
	}

	public function schema($name, $data)
	{
		return isset($data['schema']) ? $data['schema'] : array();
	}

	public function process_schema($name, $data, $table, &$fields)
	{
		return $this;
	}
//--------------------------------------------------
//END CONFIGURE

//ACCESS
//--------------------------------------------------
	public function __get($p)
	{
		switch ($p) {
			case 'view':
			case 'dir':
				return $this->$p();
			case 'js_eval':
				return $this->$p;
		}
	}

	public function __set($p, $v)
	{
		switch ($p) {
			case 'view':
			case 'dir':
				return $this->$p($v);
		}
	}

	/**
	 * Возвращает или устанавливает (если передан аргумент) текущий шаблон
	 *
	 * @param null|Templates_HTML_Template $view
	 * @return Templates_HTML_Template
	 */
	public function view($view = null)
	{
		if ($view instanceof Templates_HTML_Template) {
			$this->view = $view;
		}
		return $this->view;
	}

	public function dir($dir = null)
	{
		return is_null($dir) ? $this->dir : $this->dir = $dir;
	}

	/**
	 * Возвращает список директорий с шаблонами для типа
	 *
	 * @param $type
	 *
	 * @return array
	 * @throws Exception
	 */
	public function template_dirs($type)
	{
		$dirs = array();

		$types = array();
		$parents = Core_Types::class_hierarchy_for($this);

		foreach (array_slice($parents, 0, count($parents) - 1) as $parent) {
			$module = Core_Types::virtual_class_name_for($parent);
			$dirs[] = IO_FS::File(Core::loader()->file_path_for($module))->dir_name;

			$type = CMS_Fields::type_by_class($parent);
			if (empty($type)) {
				$type = CMS_Fields::type_by_class(Core_Types::module_name_for($parent));
			}
			if ($type && !isset($types[$type])) {
				$types[$type] = $type;
			}
		}

		foreach($types as $type) {
			$dirs[] = 'fields/'. $type;
		}

		return $dirs;
	}
//--------------------------------------------------
//EDN ACCESS

//FORM
//--------------------------------------------------
	/**
	 * Для задания своих элементов формы
	 *
	 * @param Forms_Form $form объект формы
	 * @param string     $name имя поля
	 * @param array      $data параметры, описывающие поле
	 * @return Forms_Form
	 */
	public function form_fields($form, $name, $data)
	{

		if ($langs = $this->data_langs($data)) {
			foreach ($langs as $lang => $ldata) {
				$form->input($this->name_lang($name, $lang));
			}
			return $form;
		} else {
			return $form->input($name);
		}
	}
//--------------------------------------------------
//END FORM

	public function serialized($name, $data)
	{
		return array();
	}

//LANG
//--------------------------------------------------
	/**
	 * Если возвращает true то считается что поле поддерживает многоязычность.
	 *
	 * @return bool
	 */
	public function enable_multilang()
	{
		return false;
	}

	/**
	 * Возвращает поддерживаемые языки
	 *
	 * @param array $data параметры, описывающие поле
	 * @return array|false
	 */
	public function data_langs($data)
	{
		if (!$this->enable_multilang()) {
			return false;
		}
		if (!isset($data['multilang'])) {
			return false;
		}
		if (!$data['multilang']) {
			return false;
		}
		$langs = CMS::lang()->langs();
		if (!is_array($langs)) {
			return false;
		}
		if (count($langs) < 2) {
			return false;
		}
		return $langs;
	}

	/**
	 * @param string $name имя поля
	 * @param string $lang
	 * @return string
	 */
	public function name_lang($name, $lang)
	{
		return "_lang__{$name}__{$lang}";
	}
//--------------------------------------------------
//END LANG

//ASSIGN
//--------------------------------------------------
	/**
	 * Присвоение данных из объекта в форму
	 *
	 * @param Forms_Form    $form объект формы
	 * @param DB_ORM_Entity $object
	 * @param string        $name имя поля
	 * @param array         $data параметры, описывающие поле
	 */
	public function assign_from_object($form, $object, $name, $data)
	{
		if (!$this->access($name, $data, 'assign_from_object', $object, $form)) {
			return;
		}
		$value = is_object($object) ? $object[$name] : $object;
		if ($langs = $this->data_langs($data)) {
			$values = CMS::lang()->lang_split($value);
			foreach ($langs as $lang => $ldata) {
				$_name = $this->name_lang($name, $lang);
				$form[$_name] = isset($values[$lang]) ? $values[$lang] : '';
			}
		} else {
			$form[$name] = $value;
		}
	}

	public function copy_value($from, $to, $name, $data)
	{
		$to[$name] = $from[$name];
		return $this;
	}

	/**
	 * Присвоение данных из формы в объект
	 *
	 * @param Forms_Form    $form объект формы
	 * @param DB_ORM_Entity $object
	 * @param string        $name имя поля
	 * @param array         $data параметры, описывающие поле
	 */
	public function assign_to_object($form, $object, $name, $data)
	{
		if (!$this->access($name, $data, 'assign_to_object', $object, $form)) {
			return;
		}
		if ($langs = $this->data_langs($data)) {
			$value = '';
			foreach ($langs as $lang => $ldata) {
				$_name = $this->name_lang($name, $lang);
				$_value = trim($form[$_name]);
				if ($_value != '') {
					$value .= "%LANG{{$lang}}" . $_value;
				}
			}
			$object[$name] = $value;
		} else {
			$object[$name] = $form[$name];
		}

		if (isset($data['input formats'])) {
			$this->process_input_format($form, $object, $name, $data);
		}
	}

	protected function process_input_format($form, $object, $name, $data)
	{
		$request_name = $this->input_formats_name($name, $data);
		$format_name = !empty(WS::env()->request[$request_name])
			? WS::env()->request[$request_name]
			: $data['default input format'];
		if (isset($data['input formats'][$format_name]['process'])) {
			$process = $data['input formats'][$format_name]['process'];
			$object[$name] = Core::make('Text.Process')->process($object[$name], $process);
		}
	}

	public function init_value($name, $data, $item)
	{
		$container = $this->container($name, $data, $item);
		$value = isset($data['init_value']) ? $data['init_value'] : '';
		$container->set($value);
		return $item;
	}


//--------------------------------------------------
//END ASSIGN

//CONTROLLER
//--------------------------------------------------

	public function controller($name, $data, $item = false)
	{
		if (isset($data['controller'])) {
			$controller = Core::make($data['controller'], WS::env(), WS::env()->response);
		} else {
			$controller = CMS::$current_controller;
		}
		return $controller;
	}

	/**
	 * Вызываются до срабатывания действия
	 *
	 * @param string $name имя поля
	 * @param array  $data параметры, описывающие поле
	 * @param string $action
	 * @param bool   $item
	 * @param array  $fields
	 * @return mixed
	 */
	public function on_before_action($name, $data, $action, $item = false, $fields = array())
	{
		return false;
	}

	/**
	 * Вызываются после срабатывания действия
	 *
	 * @param mixed  $result
	 * @param string $name имя поля
	 * @param array  $data параметры, описывающие поле
	 * @param string $action
	 * @param bool   $item
	 * @param array  $fields
	 * @return mixed
	 */
	public function on_after_action($result, $name, $data, $action, $item = false, $fields = array())
	{
		return false;
	}

	/**
	 * Реализация ajax действий
	 *
	 * @param string              $name имя поля
	 * @param array               $data параметры, описывающие поле
	 * @param string              $action
	 * @param false|DB_ORM_Entity $item
	 * @param array               $fields
	 * @return mixed
	 */
	public function action($name, $data, $action, $item = false, $fields = array())
	{
		if (!$this->access($name, $data, 'action', $action, $item, $fields)) {
			return Net_HTTP::forbidden();
		}
		if (method_exists($this, $m = 'action_' . $action)) {
			return $this->$m($name, $data, $action, $item, $fields);
		}
		return false;
	}

	/**
	 * Если необходимо по особенному формировать ссылку для действия, то в этом методе
	 *
	 * @param string              $name имя поля
	 * @param array               $data параметры, описывающие поле
	 * @param string              $action
	 * @param false|DB_ORM_Entity $item
	 * @param mixed               $args
	 * @return bool|string
	 */
	public function action_url($name, $data, $action, $item = false, $args = false)
	{
		return false;
	}

	/**
	 * Вызывается при обработке загрузки файлов. Здесь определяется куда и как сохранять файлы
	 *
	 * @param string        $name имя поля
	 * @param array         $data параметры, описывающие поле
	 * @param array         $uploads
	 * @param DB_ORM_Entity $item
	 * @param array         $extra
	 */
	public function process_uploads($name, $data, $uploads, $item, $extra)
	{
	}

	public function process_inserted($name, $data, $item)
	{
		Events::call('cms.fields.inserted', $name, $data, $item);
		Events::call("cms.fields.{$name}.inserted", $data, $item);

		return false;
	}

	public function action_upload($name, $data, $action, $item = false)
	{
		$files = array_pop($_POST);
		$files = is_array($files) ? $files : array($files);
		$error = '';
		$msg = '';
		if (count($files) == 0) {
			return CMS::lang()->_common->error_file_upload;
		} else {
			try {
				$res = $this->upload_files($files, $name, $data, $action, $item);
				if (!empty($res)) {
					return $res[0];
				}
			} catch (Exception $e) {
				return $e->getMessage();
			}
		}
		return CMS::lang()->_common->error_file_upload;
	}

	protected function upload_file($fobject, $name, $data, $action, $item)
	{
		$file = $fobject->file_array;
		$filename = $this->real_uploaded_filename($name, $data, $file);
		$code = $this->request('code');
		$dir = $this->dir_path($item, $code, $name, $data);
		if (!empty($file['error'])) {
			return $this->upload_error_message($file['error']);
		} else {
			if (!isset($file['tmp_name']) || empty($file['tmp_name']) || $file['tmp_name'] == 'none') {
				throw new Exception(CMS::lang()->_common->no_file_uploaded); //TODO: Exception class
			} else {
				$old = $file['tmp_name'];
				$new = "$dir/$filename";
				$valid = $this->upload_validate($name, $data, $file, $new);
				if ($valid !== true) {
					throw new Exception($valid);
				}

				$eres = Events::call('cms.fields.upload', $name, $data, $file, $new);
				if (!is_null($eres) && $eres !== true) {
					return $eres;
				}
				$eres = Events::call("cms.fields.{$name}.upload", $data, $file, $new);
				if (!is_null($eres) && $eres !== true) {
					return $eres;
				}

				if (!is_dir($dir)) {
					CMS::mkdirs($dir);
				}
				CMS::copy($old, $new);
				CMS::chmod_file($new);

				return $this->upload_return($name, $data, $new, $dir, $filename);
			}
		}
	}

	protected function upload_files($files, $name, $data, $action, $item)
	{
		$res = array();
		foreach ($files as $fobject) {
			$res = array_merge($res, (array)$this->upload_file($fobject, $name, $data, $action, $item));
		}
		return $res;
	}

	protected function upload_validate($name, $data, $file, $new)
	{
		return true;
	}

	protected function real_uploaded_filename($name, $data, $file)
	{
		if (isset($data['uploaded_filename'])) {
			$callback = $data['uploaded_filename'];
			$result = call_user_func($callback, trim($file['name']), $file, $name, $data);
			if ($result) {
				return $result;
			}
		}
		return $this->uploaded_filename($name, $data, $file);
	}

	protected function uploaded_filename($name, $data, $file)
    {
		$filename = CMS::translit(preg_replace('{[\s+]+}', '-', trim($file['name'])));
		$filename = preg_replace('{[^-0-9a-z_\.]}i', '_', $filename);
		return $filename;
	}

	protected function upload_return($name, $data, $new_file, $dir, $filename)
	{
		return 'success';
	}

	/**
	 * @param CMS_ORM_Entity $item
	 * @param string         $code
	 * @param string         $name имя поля
	 * @param array          $data параметры, описывающие поле
	 * @return mixed|string
	 */
	public function dir_path($item, $code, $name, $data)
	{
		$item_id = $item ? $item->id() : 0;
		$private = isset($data['private']) ? $data['private'] : false;
		$key = "$name-$item_id-$code-$private";
		if ($item->mapper) {
			$key .= '-' . reset($k = $item->mapper->options['table']);
		}
		if (isset($this->dir_paths[$key])) {
			return $this->dir_paths[$key];
		}
		if (empty($item_id)) {
			$dir = CMS::temp_dir() . "/dir-$code";
		} else {
			$dir = $item->homedir(isset($data['private']) && $data['private']);
			if ($dir == '') {
				die;
			}
			if ($dir[0] != '.' && $dir[0] != '/') {
				$dir = "./$dir";
			}
		}
		$dir .= "/$name";
		return $this->dir_paths[$key] = $dir;
	}
//--------------------------------------------------
//END CONTROLLER

//SUPPORT
//--------------------------------------------------

	public function search_subfield($name, $data, $field)
	{
		return false;
	}

	public function input_formats_name($name, $data)
	{
		return $name . '_formats';
	}

	public function url_class()
	{
		return 'field-url-' . WS::env()->request->id;
	}

	protected function punset($data)
	{
		$args = func_get_args();
		foreach ($args as $arg)
			if (is_string($arg)) {
				if (isset($data[$arg])) {
					unset($data[$arg]);
				}
			}
		return $data;
	}

	/**
	 * Удаление параметров, которые не должны отображаться в data-атрибуте input-поля
	 *
	 * @param array $data параметры, описывающие поле
	 * @return array
	 */
	protected function stdunset($data)
	{
		if (isset($data['value']) && !$data['value']) {
			unset($data['value']);
		}
		if (empty($data)) {
			return array();
		}
		$keys = array_filter(array_keys($data), array($this, 'tag_attrs_filter'));
		$attrs = array();
		foreach ($keys as $key) {
			if (isset($data[$key])) {
				$attrs[$key] = $data[$key];
			}
		}
		return $attrs;
	}

	protected function tag_attrs_filter($attr)
	{
		return Core_Strings::starts_with($attr, 'data-') ||
			Core_Strings::starts_with($attr, 'on') ||
			in_array($attr, $this->tag_attrs())
		;
	}

	protected function tag_attrs()
	{
		return self::$tag_attrs;
	}

	public function user()
	{
		if (Core::is_cli()) {
			return 'cli';
		}
		if (CMS::admin()) {
			return 'admin-' . WS::env()->admin_auth->user->login;
		}
		if (WS::env()->auth->user) {
			return WS::env()->auth->user->id;
		}
		return false;
	}

	public function temp_code()
	{
		if (!$this->temp_code) {
			$u = $this->user();
			if (!$u) {
				die;
			}
			$t = date('YmdHis');
			$r = rand(1111, 9999);
			$this->temp_code = "$t-$r-$u";
		}
		return $this->temp_code;
	}

	protected function upload_error_message($e)
	{
		switch ((int)$e) {
			case 1:
				return 'The uploaded file exceeds the upload_max_filesize directive in php.ini';
			case 2:
				return 'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form';
			case 3:
				return 'The uploaded file was only partially uploaded';
			case 4:
				return 'No file was uploaded.';
			case 6:
				return 'Missing a temporary folder';
			case 7:
				return 'Failed to write file to disk';
			case 8:
				return 'File upload stopped by extension';
		}
		return "Unknown error ($e)";
	}

	/**
	 * @param Templates_HTML_Template $template
	 * @param string                  $field_name
	 * @param string                  $type
	 * @param bool                    $url_class
	 * @param null                    $method_name
	 */
	protected function js_injection($template, $field_name, $type, $url_class = false, $method_name = null)
	{
		$selector = '.field-' . $field_name;
		if ($url_class) {
			$selector = '.' . $url_class . $selector;
			$template->with('url_class', $url_class);
		}

		$call = $method_name ? "$type.$method_name" : $type;
		$code = "; $(function() { $('$selector').each(
					function() {
						TAO.fields.$call($(this));
						}
		)});";
		$template->append_to('js', $code);
	}

	public function use_styles()
	{
		return $this->view->use_styles(func_get_args());
	}

	public function use_scripts()
	{
		return $this->view->use_scripts(func_get_args());
	}

	public function request($index = null)
	{
		return is_null($index) ? WS::env()->request : WS::env()->request[(string)$index];
	}

	public function access_default($name, $data, $action, $item = null, $args = array())
	{
		//$args = array_slice(func_get_args(), 3);
		$rc = Events::call('cms.fields.access', $name, $data, $action, $item);
		if (!is_null($rc)) {
			return $rc;
		}
		$rc = Events::call('cms.fields.access.' . $action, $name, $data, $item);
		if (!is_null($rc)) {
			return $rc;
		}
		return true;
	}

	public function access($name, $data, $action, $item = null)
	{
		$args = array_slice(func_get_args(), 4);
		if (isset($data['access']) && Core_Types::is_callable($data['access'])) {
			return Core::invoke($data['access'], array($this, $name, $data, $action, $item, $args));
		}
		return $this->access_default($name, $data, $action, $item, $args);
	}

//--------------------------------------------------
//END SUPPORT

// WIDGET:
//--------------------------------------------------
	/**
	 * Возвращает путь к обрамлению с именем $layout
	 *
	 * @param string|false $layout
	 * @param array        $data параметры, описывающие поле
	 * @return bool|mixed|string
	 * @throws Exception
	 */
	public function layout($layout = false, $data = array())
	{
		if ($layout && IO_FS::exists($layout)) {
			return $layout;
		}
		if (isset($data['layout'])) {
			if(IO_FS::exists($data['layout'])) {
				return $data['layout'];
			}
			$layout = $data['layout'];
		}
		return $this->template($data, $layout ? "layout-$layout" : 'layout');
	}

	/**
	 * Возвращает путь к шаблону с именем $name.
	 * Сначала шаблон ищется в текущем каталоге (где находится модуль), затем в библиотечных шаблонах.
	 * Для указания произвольного места для шаблона можно переопределить метод dir.
	 *
	 * @param array  $data параметры, описывающие поле
	 * @param string $name имя поля
	 * @return mixed|string
	 * @throws Exception
	 */
	public function template($data = array(), $name = 'template')
	{
		static $template = array();

		if (Templates::is_absolute_path($name)) {
			return $name;
		}

		if (isset($data['template'])) {
			$data_templates = is_string($data['template']) ? array('template' => $data['template']) : $data['template'];
			if (isset($data_templates[$name]) && IO_FS::exists($data_templates[$name])) {
				return $data_templates[$name];
			}
		}

		$type = isset($data['type']) ? $data['type'] : 'input';
		$key = $type . '_' . $name;
		if (isset($template[$key])) {
			return $template[$key];
		}

		$dirs = $this->template_dirs($type);
		foreach($dirs as $dir) {
			$view = Templates_HTML::Template($dir .'/'. $name);
			if ($view->exists()) {
				return $template[$key] = $view->path;
			}
		}

		$view = Templates_HTML::Template('fields/' . $name);
		if ($view->exists()) {
			return $template[$key] = $view->path;
		}

		return $template[$key] = $name;
	}

	public function caption_content($name, $data)
	{
		return false;
	}

	/**
	 * @see CMS_Fields_AbstractField::render()
	 *
	 * @param string $name имя поля
	 * @param array  $data параметры, описывающие поле
	 * @param string $template
	 * @param array  $parms
	 * @return string
	 * @throws Exception
	 */
	public function draw($name, $data, $template = 'template', $parms = array())
	{
		return $this->render($name, $data, $template, $parms);
	}

	/**
	 * Отображение шаблона с именем $template и параметрами $parms
	 *
	 * @param string $name имя поля
	 * @param array  $data параметры, описывающие поле
	 * @param string $template
	 * @param array  $parms
	 * @return string
	 * @throws Exception
	 */
	public function render($name, $data, $template = 'template', $parms = array())
	{
		if ($this->access($name, $data, 'render', $data['__item'], $template)) {
			$template = $this->create_template($name, $data, $template, $parms);
			$this->layout_preprocess($template, $name, $data);
			return $template->render();
		}
		return '';
	}

	/**
	 * Создание шаблона для для методов draw и render
	 *
	 * @param string $name имя поля
	 * @param array  $data параметры, описывающие поле
	 * @param string $template
	 * @param array  $parms
	 * @return Templates_HTML_Template
	 * @throws Exception
	 */
	public function create_template($name, $data, $template = 'template', $parms = array())
	{
		$template = $this->template($data, $template);
		//FIXME
		$t = !empty($this->view) ? $this->view->spawn($template) : Templates_HTML::Template($template);

		$t->with(array('template' => $template));
		$t->with($parms);
		$this->preprocess($t, $name, $data);
		return $t;
	}

	/**
	 * Создание шаблона для для методов draw_in_layout и render_in_layout
	 *
	 * @param string $name имя поля
	 * @param array  $data параметры, описывающие поле
	 * @param bool   $layout
	 * @param string $template
	 * @param array  $parms
	 * @return Templates_HTML_Template
	 * @throws Exception
	 */
	public function create_layout($name, $data, $layout = false, $template = 'template', $parms = array())
	{
		$layout = $this->layout($layout, $data);
		$template = $this->template($data, $template);
		//FIXME
		$l = !empty($this->view) ? $this->view->spawn($layout) : Templates_HTML::Template($layout);
		$l->no_duplicates_in('js')->no_duplicates_in('css');

		$l = $l->with(array(
				'template' => $template,
				'layout' => $layout,
			)
		);
		$l->with($parms);
		$this->layout_preprocess($l, $name, $data);
		$this->preprocess($l, $name, $data);
		return $l;
	}

	/**
	 * Отображение на экране шаблона с именем $template внутри обрамления $layout
	 *
	 * @param string $name имя поля
	 * @param array  $data параметры, описывающие поле
	 * @param bool   $layout
	 * @param string $template
	 * @param array  $parms
	 * @throws Exception
	 */
	public function draw_in_layout($name, $data, $layout = false, $template = 'template', $parms = array())
	{
		print $this->render_in_layout($name, $data, $layout);
	}

	/**
	 * Рендер шаблона с именем $template внутри обрамления $layout
	 *
	 * @param string $name имя поля
	 * @param array  $data параметры, описывающие поле
	 * @param bool   $layout
	 * @param string $template
	 * @param array  $parms
	 * @return string|Templates_HTML_Template
	 * @throws Exception
	 */
	public function render_in_layout($name, $data, $layout = false, $template = 'template', $parms = array())
	{
		if ($this->access($name, $data, 'render_in_layout', $data['__item'], $layout, $template)) {
			return $this->create_layout($name, $data, $layout, $template, $parms)->render();
		}
		return '';
	}

	public function get_item($name, $data)
	{
		return $data['__item'];
	}

	public function tagparms($name, $data)
	{
		$r = array_merge(isset($data['tagparms']) ? $data['tagparms'] : array(), $this->stdunset($data));
		$this->validator_tagparms($name, $data, $r);
		return $r;
	}

	protected function get_form_for_template($template, $name, $data)
	{
		$form = $template->helpers['forms']->form;
		if (!$form && isset($template->parms['form'])) {
			$form = $template->helpers['forms']->form = $template->parms['form'];
		}
		if (!$form) {
			$form = $this->view->helpers['forms']->form;
		}
		// in same case (ajax processing), $form === null is normal
		// if (!$form) {
		// 	throw new RuntimeException('CMS.Fields: render without form');
		// }
		return $form;
	}

	/**
	 * Переопределение/добавление параметров шаблона, можно менять помощью методов update_parm и replace_parm
	 *
	 * @param Templates_HTML_Template $template экземпляр шаблона
	 * @param string                  $name     имя поля
	 * @param array                   $data     параметры, описывающие поле
	 * @return Templates_HTML_Template
	 */
	protected function preprocess($template, $name, $data)
	{
		$form = $this->get_form_for_template($template, $name, $data);
		$tagparms = $this->tagparms($name, $data);
		$tagparms['data-field-name'] = $name;
		$item = isset($data['__item']) ? $data['__item'] : (isset($data['item']) ? $data['item'] : false);
		$item_id = isset($data['__item_id']) ? $data['__item_id'] : 0;
		$template->with(array(
				'form' => $form,
				'tagparms' => $tagparms,
				'name' => $name,
				'_name' => $name,
				'data' => $data,
				'type_object' => $this,
				'item' => $item,
				'item_id' => $item_id
			)
		);
		if (isset($data['preprocess'])) {
			Core::invoke($data['preprocess'], array($template, $name, $data));
		}
		return $template->allow_filtering(false);
	}

	protected function current_lang()
	{
		$current_lang = CMS::$default_lang;
		if (isset($_COOKIE['admin-field-lang'])) {
			$current_lang = $_COOKIE['admin-field-lang'];
		}
		return $current_lang;
	}

	/**
	 * Для переопределения/добавления параметров обрамления, а так же для добавления css и js файлов
	 *
	 * @param Templates_HTML_Template $l
	 * @param string                  $name имя поля
	 * @param array                   $data параметры, описывающие поле
	 * @return Templates_HTML_Template
	 */
	protected function layout_preprocess($l, $name, $data)
	{
		$form_name = $l->helpers['forms']->form->name;
		$this->attach($l, $name, $data);
		if (!isset($data['caption'])) {
			$data['caption'] = '';
		}
		$caption = trim(CMS::lang($data['caption']));
		$rcaption = '';
		if (isset($data['rcaption'])) {
			$caption = '';
			$rcaption = trim(CMS::lang($data['rcaption']));
		}
		if ($caption == '') {
			$caption = '&nbsp;';
		} else {
			$caption .= ':';
		}

		Events::call("cms.fields.$form_name.$name.caption", $caption, $rcaption);
		Events::call("cms.fields.$form_name.caption", $caption, $rcaption);

		$ccont = $this->caption_content($name, $data);
		if (!$ccont) {
			$ccont = $l->forms->label($name, $caption, array('class' => "label-$name left"));
		}

		if (isset($data['allow select format']) && $data['allow select format'] && isset($data['input formats'])) {
			$select_formats = array();
			foreach ($data['input formats'] as $name => $f) {
				$select_formats[$name] = $f['display name'];
			}
			$l->update_parm('selected_formats', $select_formats);
		}

		$l->with(array(
				'caption' => $caption,
				'rcaption' => $rcaption,
				'current_lang' => $this->current_lang(),
				'ccont' => $ccont
			)
		);

		if (isset($data['layout preprocess'])) {
			Core::invoke($data['layout preprocess'], array($l, $name, $data));
		}
		return $l;
	}

	protected function attach($template, $name, $data)
	{
		if (isset($data['attach'])) {
			$method_by_name = array('js' => 'use_scripts', 'css' => 'use_styles');
			foreach ($data['attach'] as $name => $data) {
				$template->{$method_by_name[$name]}($data);
			}
		}
	}

//END WIDGET
//--------------------------------------------------

//VALIDATION
//--------------------------------------------------

	public function validator($form)
	{
		if (!$form->validator) {
			$validator = Validation::Validator();
			$form->validate_with($validator);
		}
		return $form->validator;
	}

	public function form_validator($form, $name, $data)
	{
		if (isset($data['validate_presence'])) {
			$this->validator($form)->validate_presence_of($name, $data['validate_presence']);
		}
		if (isset($data['validate_range_from']) || isset($data['validate_range_to'])) {
			$from = isset($data['validate_range_from']) ? $data['validate_range_from'] : 0;
			$to = isset($data['validate_range_to']) ? $data['validate_range_to'] : PHP_INT_MAX;
			$message = isset($data['validate_range_message']) ? $data['validate_range_message'] : "$name!";
			$this->validator($form)->validate_range_of($name, $from, $to, $message);
		}
		if (isset($data['validate_email'])) {
			$this->validator($form)->validate_email_for($name, $data['validate_email']);
		}
		if (isset($data['validate_match']) && isset($data['validate_match_message'])) {
			$this->validator($form)->validate_format_of($name, $data['validate_match'], $data['validate_match_message']);
		}
		if (isset($data['validate_confirmation']) && isset($data['validate_confirmation_message'])) {
			$this->validator($form)->validate_confirmation_of($name, $data['validate_confirmation'], $data['validate_confirmation_message']);
		}
		return $form;
	}

	public function validate_email($email)
	{
		return Core_Regexps::match('{' . $this->email_regexp() . '}', $email);
	}

	public function email_regexp()
	{
		return '^$|^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}$';
	}

	public function validator_tagparms($name, $data, &$tagparms)
	{
		$rc = false;
		if (isset($data['validate_ajax'])) {
			$rc = true;
			$tagparms['data-validate-ajax'] = trim($data['validate_ajax']);
		}
		if (isset($data['validate_presence'])) {
			$rc = true;
			$tagparms['data-validate-presence'] = htmlspecialchars($data['validate_presence']);
		}
		if (isset($data['validate_confirmation'])) {
			$rc = true;
			$tagparms['data-validate-confirmation'] = trim($data['validate_confirmation']);
			$tagparms['data-validate-confirmation-message'] = htmlspecialchars($data['validate_confirmation_message']);
		}
		if (isset($data['validate_email'])) {
			$rc = true;
			$tagparms['data-validate-match'] = $this->email_regexp();
			$tagparms['data-validate-match-message'] = $data['validate_email'];
			$tagparms['data-validate-match-mods'] = '';
		}
		if (isset($data['validate_match']) && isset($data['validate_match_message'])) {
			$rc = true;
			$match = trim($data['validate_match']);
			$mods = '';
			if ($match[0] == '/' || $match[0] == '{') {
				$mods = '';
				if ($m = Core_Regexps::match_with_results('!^(?:{|/)(.+)(?:}|/)([a-z]*)$!', $match)) {
					$match = $m[1];
					$mods = $m[2];
				}
			}
			$tagparms['data-validate-match'] = $match;
			$tagparms['data-validate-match-mods'] = $mods;
			$tagparms['data-validate-match-message'] = htmlspecialchars($data['validate_match_message']);
		}
		if ($rc) {
			$class = isset($tagparms['class']) ? $tagparms['class'] : '';
			$class = trim("$class validable");
			$tagparms['class'] = $class;
		}
		return $rc;
	}

//--------------------------------------------------
//END VALIDATION

//DISPLAY
//--------------------------------------------------
	/**
	 * Для переопределеиния отображения в списке таблицы
	 *
	 * @param mixed  $value
	 * @param string $name имя поля
	 * @param array  $data параметры, описывающие поле
	 * @param mixed  $row
	 * @return CMS_Lang|mixed|string
	 */
	public function view_value($value, $name, $data, $row = false)
	{
		if (is_object($value)) {
			$value = $value[$name];
		}
		if (is_string($value) && $value != '' && $this->enable_multilang()) {
			$value = CMS::lang($value);
		}
		if (isset($data['view_preprocess']) && is_callable($data['view_preprocess'])) {
			$value = call_user_func($data['view_preprocess'], $value, $row);
		}
		return $value;
	}

	protected function container_class()
	{
		$parents = Core_Types::class_hierarchy_for($this);
		foreach ($parents as $parent) {
			if ($parent == 'CMS_Fields_AbstractField') {
				return 'CMS.Fields.ValueContainer';
			}
			$class = "{$parent}_ValueContainer";
			if (class_exists($class)) {
				return $class;
			}
		}
		return 'CMS.Fields.ValueContainer';
	}

	/**
	 * @param string         $name имя поля
	 * @param array          $data параметры, описывающие поле
	 * @param CMS_ORM_Entity $item
	 *
	 * @return CMS_Fields_ValueContainer
	 */
	public function container($name, $data, $item)
	{
		$class = $this->container_class();
		if (isset($data['container']) && Core_Types::is_subclass_of('CMS.Fields.ValueContainer', $data['container'])) {
			$class = $data['container'];
		}
		return Core::make($class, $name, $data, $item, $this);
	}
//--------------------------------------------------
//END DISPLAY
}

class CMS_Fields_Exception extends Core_Exception {}

