<?php

/**
 * @package CMS\Fields\Types\Relations
 */
class CMS_Fields_Types_Relations extends CMS_Fields_AbstractField implements Core_ModuleInterface
{
	static $cache = array();

	public static function table_name($name, $table, $data = array())
	{
		if (isset($data['table_name'])) {
			return $data['table_name'];
		}
		return "{$table}_{$name}_relations";

	}

	public static function item_id_name($name, $table, $data = array())
	{
		if (isset($data['item_id_name'])) {
			return $data['item_id_name'];
		}
		return "item_id";

	}

	public static function rel_id_name($name, $table, $data = array())
	{
		if (isset($data['rel_id_name'])) {
			return $data['rel_id_name'];
		}
		return "rel_id";

	}

	public static function get_rels($name, $table, $id, $data = array())
	{
		$cn = "rels-$table-$name-$id";
		if (isset(self::$cache[$cn])) {
			return self::$cache[$cn];
		}
		$table_name = self::table_name($name, $table, $data);
		$item_name = self::item_id_name($name, $table, $data);
		$rel_name = self::rel_id_name($name, $table, $data);
		$rels = array();
		$cursor = CMS::orm()->connection
			->prepare("SELECT rels.{$rel_name} id FROM {$table_name} rels WHERE rels.{$item_name}=:item_id")
			->bind(array('item_id' => $id))
			->execute();
		foreach ($cursor as $row) {
			$rels[$row['id']] = $row['id'];
		}
		self::$cache[$cn] = $rels;
		return $rels;
	}

	protected function items2fields($items, $form, $name, $data)
	{
		foreach ($items as $id => $item) {
			$form->checkbox("{$name}{$id}");
			if ($item->childs) {
				$this->items2fields($item->childs, $form, $name, $data);
			}
		}
	}

	public function form_fields($form, $name, $data)
	{
		$items = isset($data['items']) ? CMS::items_for_select($data['items']) : false;
		if ($items) {
			$this->items2fields($items, $form, $name, $data);
		} else {
			return parent::form_fields($form, $name, $data);
		}
	}

	public function assign_from_object($form, $item, $name, $data)
	{
		$id = (int)$item->id();
		if ($id > 0) {
			if ($item->mapper) {
				$table = $item->mapper->options['table'][0];
				$rels = self::get_rels($name, $table, $id, $data);
				foreach ($rels as $rel) {
					$_name = "{$name}{$rel}";
					if (!isset($form[$_name])) {
						$form->input($_name);
					}
					if (isset($form[$_name])) {
						$form[$_name] = 1;
					}
				}
			}
		}
	}

	public function assign_to_object($form, $item, $name, $data)
	{

		$id = (int)$item->id();
		if ($id == 0) {
			return;
		}
		$values = array();
		foreach ($form->fields as $k => $v) {
			if ($m = Core_Regexps::match_with_results('{^' . $name . '(\d+)$}', $k)) {
				$rel_id = (int)$m[1];
				if ($rel_id && $form[$k]) {
					$values[] = $rel_id;
				}
			}
		}
		$this->set($values, $name, $data, $item);
	}

	public function set($values, $name, $data, $item)
	{
		if(!is_array($values)) {
			$values = array($values);
		}

		$id = (int) $item->id();
		if ($id == 0) {
			return false;
		}

		if ($item->mapper) {
			$table = $item->mapper->options['table'][0];
			$table_name = self::table_name($name, $table, $data);
			$item_name = self::item_id_name($name, $table, $data);
			$rel_name = self::rel_id_name($name, $table, $data);
			CMS::orm()->connection
				->prepare("DELETE FROM {$table_name} WHERE {$item_name}=:item_id")
				->bind(array('item_id' => $id))
				->execute();
			foreach ($values as $value) {
				CMS::orm()->connection
					->prepare("INSERT INTO {$table_name} SET {$item_name}=:item_id, {$rel_name}=:rel_id")
					->bind(array(
							'item_id' => $id,
							'rel_id' => (int)$value,
						)
					)
					->execute();
			}
		}
		return $this;
	}

	protected function preprocess($template, $name, $data)
	{
		$t = parent::preprocess($template, $name, $data);
		return $t->with('items', isset($data['__items']) ? $data['__items'] : CMS::items_for_select($data['items']));
	}

	public function process_schema($name, $data, $table)
	{
		$rels = self::table_name($name, $table, $data);
		$item_name = self::item_id_name($name, $table, $data);
		$rel_name = self::rel_id_name($name, $table, $data);
		$fields = array(
			$item_name => array(
				'sqltype' => 'int index',
			),
			$rel_name => array(
				'sqltype' => 'int index',
			),
		);
		CMS_Fields::process_schema($rels, $fields);
	}
}

class CMS_Fields_Types_Relations_ValueContainer extends CMS_Fields_ValueContainer
{
	public function ids()
	{
		$table = $this->item->mapper->options['table'][0];
		$ids = $this->type->get_rels($this->name, $table, $this->item->id, $this->data);
		return $ids;
	}

	public function render()
	{
		return 'rels';
	}

	public function set($values)
	{
		$this->type->set($values, $this->name, $this->data, $this->item);
		return $this;
	}
}
