<?php

Core::load('Text.Process');

/**
 * @package CMS\Fields\Types\Textarea
 */
class CMS_Fields_Types_Textarea extends CMS_Fields_AbstractField implements Core_ModuleInterface
{

	const VERSION = '0.0.0';

	public function enable_multilang()
	{
		return true;
	}

	public function form_fields($form, $name, $data)
	{
		$rc = parent::form_fields($form, $name, $data);
		if (isset($data['value'])) {
			$form[$name] = $data['value'];
		}
		return $rc;
	}

	protected function stdunset($data)
	{
		$res = parent::stdunset($data);
		return $this->punset($res, 'value', 'htmlpurifier');
	}

	public function preprocess($template, $name, $data)
	{
		parent::preprocess($template, $name, $data);
		$parms = $template->parms;
		$class = isset($parms['tagparms']['class']) ? $parms['tagparms']['class'] : '';
		$use_tab_key = true;
		if (isset(WS::env()->config->fields->textarea_tab_key)) {
			$use_tab_key = WS::env()->config->fields->textarea_tab_key;
		}
		if (isset($data['use_tab_key'])) {
			$use_tab_key = $data['use_tab_key'];
		}
		if ($use_tab_key) {
			$template->use_scripts(CMS::stdfile_url('scripts/jquery/tabby.js'));
			$class .= ' use-tab-key';
		}
		$class = trim($class);
		$template->update_parm('tagparms', array('class' => $class));
		if (empty($parms['tagparms']['style'])) {
			$template->update_parm('tagparms', array('style' => 'width: 300px;height:100px;'));
		}
		return $template;
	}

}

class CMS_Fields_Types_Textarea_ValueContainer extends CMS_Fields_ValueContainer
{
	protected $value_processors = array();

	public function __construct($name, $data, $item, $type)
	{
		parent::__construct($name, $data, $item, $type);
		$this->setup_value_processors();
	}

	public function teaser()
	{
		$spliter = $this->get_spliter();
		if (empty($spliter)) {
			return '';
		}
		$value = $this->value();
		$delimiter = strpos($value, $spliter);
		if ($delimiter !== false) {
			$teaser = substr($value, 0, $delimiter);
			if (!empty($teaser)) {
				$teaser = $this->prepare_value($teaser);
			}
			return $teaser;
		} else {
			return $this->prepare_value($value);
		}
		return '';
	}

	protected function is_htmlpurifier()
	{
		return isset($this->data['htmlpurifier']) && $this->data['htmlpurifier'];
	}

	/**
	 * @return string
	 */
	public function value()
	{
		$value = parent::value();
		return $value ? CMS::lang($value, $this->forced_lang) : '';
	}

	/**
	 * @param CMS_Lang|string|null $lang
	 * @return string
	 */
	public function multilang_value($lang = null)
	{
		return $this->parse_multilang_value($this->item->{$this->name}, $lang);
	}

	public function get_spliter()
	{
		return isset($this->data['teaser split']) ? $this->data['teaser split'] : null;
	}

	public function has_spliter()
	{
		$spliter = $this->get_spliter();
		if ($spliter) {
			return (boolean)strpos(parent::value(), $spliter);
		}
		return false;
	}

	public function clean_value()
	{
		$value = $this->value();
		if ($this->has_spliter()) {
			$find = preg_quote($this->get_spliter());
			$value = preg_replace("!(<br/?>)?{$find}(<br/?>)?!", '', $value);
		}
		$value = $this->prepare_value($value);
		return $value;
	}

	protected function prepare_value($value = false)
	{
		if ($value === false) {
			$value = $this->value();
		}

		return Text_Process::process($value, $this->value_processors());
	}

	public function nl2br($value)
	{
		return nl2br($value);
	}

	public function render()
	{
		return $this->clean_value();
	}

	public function value_processors()
	{
		return $this->value_processors;
	}

	public function set_value_processors($processors)
	{
		$this->value_processors = $processors;
		return $this;
	}

	public function add_value_processors($processors)
	{
		if (is_array($processors)) {
			$this->value_processors = array_merge($this->value_processors, $processors);
		} else {
			$this->value_processors[] = $processors;
		}
		return $this;
	}

	protected function setup_value_processors()
	{
		$this->set_value_processors(isset($this->data['value_processors']) ? $this->data['value_processors']
			: $this->default_value_processors());

		if ($this->is_htmlpurifier()) {
			$this->add_value_processors('htmlpurifier');
		}
	}

	protected function default_value_processors()
	{
		return array('nl2br');
	}
}
