<?php
/**
 * @package CMS\Controller\Stockroom
 */

Core::load('CMS.Controller.Table');

class CMS_Stockroom_Controller extends CMS_Controller_Table
{

	protected $title_list = 'Библиотека компонентов';
	protected $norows = 'Нет ни одного компонента';
	protected $all_tags_title = 'Все компоненты';
	protected $can_add = false;
	protected $can_delete = false;
	protected $can_edit = false;
	protected $list_style = 'lent';
	protected $per_page = 10;
	protected $tags = array();
	protected $error_message;
	protected $manager;

	public function setup()
	{
		$this->manager = CMS_Stockroom::manager();
		$this->manager->set_repository_name($_GET['repository']);
		$this->tags = $this->manager->send('tags.json');
		return parent::setup()->render_defaults('connect_error', 'tags');
	}

	public function __get($name)
	{
		if ($name == 'connect_error') {
			return $this->connect_error_message();
		}
		return parent::__get($name);
	}

	public function __isset($name)
	{
		if ($name == 'connect_error') {
			$err = $this->connect_error_message();
			return !empty($err);
		}
		return parent::__isset($name);
	}

	public function connect_error_message()
	{
		if ($this->error_message) {
			return $this->error_message;
		}
		$error = $this->manager->get_error();
		if (!$error) {
			return false;
		}
		switch($error['method']) {
			case 'get_query':
				$message = "<p>Ошибка подключения к <a href='{$error['url']}'>репозиторию</a></p><p class='status'>";
				if ($error['status'] > 0) {
					$message .= "{$error['status']}: ";
				}
				$message .= "{$error['response']->status->message}</p>";
				return $message;
			default:
				return false;
		}
	}

	protected function new_object()
	{
		return $this->manager->make_item();
	}

	protected function templates_dir()
	{
		return CMS::views_path('admin/stockroom');
	}

	protected function filters()
	{
		return array('tag', 'search', 'repository', 'not_tag');
	}

	protected function filters_form()
	{
		return array(
			'search' => array(
				'caption' => 'Поиск',
				'style' => 'width:150px',
			),
		);
	}

	protected function force_filters()
	{
		return $this->force_filters = array(
			'not_packages' => 1
		);
	}

	protected function count_all($parms)
	{
		$count = $this->manager->components_count($parms);
		if ($count !== false) {
			return $count;
		}
		$this->error_message = "<p>Некорректный ответ от <a href='{$this->manager->get_last_url()}'>репозитория</a></p>";
		return 0;
	}

	protected function select_all($parms)
	{
		$o = $this->manager->components_list($parms);
		if ($o !== false) {
			return $o;
		}
		$this->error_message = "<p>Некорректный ответ от <a href='{$this->manager->get_last_url()}'>репозитория</a></p>";
		return array();
	}

	protected function action_updok()
	{
		$item = $this->load_item($this->id);
		if (!$item) {
			return $this->page_not_found();
		}
		$session = $this->request->session();
		$updated = isset($session['stockroom/updated']) ? $session['stockroom/updated'] : array();
		if (!is_array($updated)) {
			$updated = array();
		}
		$not_updated = isset($session['stockroom/not_updated']) ? $session['stockroom/not_updated'] : array();
		if (!is_array($not_updated)) {
			$not_updated = array();
		}
		return $this->render('updok', array(
				'item' => $item,
				'list_url' => $this->action_url('list', $this->page),
				'list_button_caption' => $this->button_list(),
				'updated' => $updated,
				'not_updated' => $not_updated,
			)
		);
	}

	protected function action_delete()
	{
		$item = $this->load_item($this->id);
		if (!$item) {
			return $this->page_not_found();
		}
		if ($item->delete()) {
			return $this->redirect_to($this->list_url($this->page));
		} else {
			print('Error');
		}
	}

	protected function action_update()
	{
		$item = $this->load_item($this->id);
		if (!$item) {
			return $this->page_not_found();
		}
		$error = false;

		if (!$item->installed()) {
			$error = 'Этот компонент еще не установлен. Попробуйте сначала установить его.';
		} elseif ($item->installed() == $item['version']) {
			$error = 'Уже установлена последняя версия этого компонента. Обновление не требуется.';
		} else {
			if ($this->env->request->method == 'post') {
				$files = $this->manager->get_install_pack($item);
				if (is_string($files)) {
					$error = $files;
				} else {
					$updated = array();
					$not_updated = array();
					if ($result = $this->manager->update_package_files($item, $files)) {
						$updated = $result['updated'];
						$not_updated = $result['not_updated'];
					} else {
						$error = $this->manager->get_last_error();
					}
					$session = $this->request->session();
					$session['stockroom/updated'] = $updated;
					$session['stockroom/not_updated'] = $not_updated;
					return $this->redirect_to($this->action_url('updok', $item));
				}
			}
		}

		Events::call('cms.stockroom.after_update', $item, $error);

		return $this->render('update', array(
				'item' => $item,
				'error' => $error,
				'list_url' => $this->action_url('list', $this->page),
				'list_button_caption' => $this->button_list(),
			)
		);
	}

	protected function load_item($id)
	{
		return $this->manager->load_item($id);
	}

	protected function action_instok()
	{
		$item = $this->load_item($this->id);
		if (!$item) {
			return $this->page_not_found();
		}

		return $this->render('instok', array(
				'item' => $item,
				'list_url' => $this->action_url('list', $this->page),
				'list_button_caption' => $this->button_list(),
			)
		);
	}

	protected function action_install()
	{
		$item = $this->load_item($this->id);
		if (!$item) {
			return $this->page_not_found();
		}
		$error = false;

		if (!$this->manager->validate_package_intall($item)) {
			$error = $this->manager->get_last_error();
		}
		if ($this->env->request->method == 'post') {
			$files = $this->manager->get_install_pack($item);
			if (is_string($files)) {
				$error = $files;
			} else {
				if ($this->manager->install_package_files($item, $files)) {
					return $this->redirect_to($this->action_url('instok', $item));
				} else {
					$error = $this->manager->get_last_error();
				}
			}
		}

		Events::call('cms.stockroom.after_install', $item, $error);

		return $this->render('install', array(
				'item' => $item,
				'error' => $error,
				'list_url' => $this->action_url('list', $this->page),
				'list_button_caption' => $this->button_list(),
			)
		);
	}

}