<?php

Core::load('IO.Arc');

class CMS_Stockroom_Manager implements Core_ModuleInterface
{

	protected $repositories = array(
		'tao' => array(
			'url' => 'http://cms.techart.ru/components/',
			'core_download_url' => 'http://cms.techart.ru/tao/download/core/',
			'core_version_url' => 'http://cms.techart.ru/tao/version/',
			'title' => 'Текарт',
		),
	);

	protected $repository_name = null;

	protected $connect_error = null;

	protected $curl_options = array(
		CURLOPT_TIMEOUT => 5,
	);

	protected $info_dir = '../app/components/.info';

	protected $last_url = null;

	protected $last_error = null;

	protected function __construct()
	{
	}

	public static function instance()
	{
		static $instance = null;
		if (null === $instance) {
			$instance = new static();
		}
		return $instance;
	}

	public function configure($options = array())
	{
		foreach ($options as $key => $value) {
			if (isset($this->$key)) {
				$this->$key = $value;
			}
		}
		return $this;
	}

	public function get_info_dir()
	{
		return $this->info_dir;
	}

	public function get_curl_options()
	{
		return $this->curl_options;
	}

	public function get_last_url()
	{
		return $this->last_url;
	}

	public function get_last_error()
	{
		$e = $this->last_error;
		$this->last_error = null;
		return $e;
	}

	public function send($to, $parms = array(), $body = null, $method = 'get', $options = array())
	{
		$url = $this->rep_query($to, $parms);
		return $this->get_query($url, $body, $method, $options);
	}

	public function rep_query($uri, $parms = array())
	{
		$url = $this->current_repository_url();
		if (!$url) {
			throw new CMS_Stockroom_Exception('CMS.Stockroom: repositories not found!');
		}
		$q = $uri . '?site=' . $_SERVER['HTTP_HOST'];
		foreach ($parms as $k => $v) {
			$v = urldecode($v);
			$q .= "&{$k}={$v}";
		}
		$url .= $q;
		$this->last_url = $url;
		return $url;
	}

	public function current_repository_url()
	{
		$rep = $this->current_repository();
		if (!$rep) {
			return false;
		}
		return $this->repositories[$rep]['url'];
	}

	public function set_repository_name($name)
	{
		$this->repository_name = $name;
		return $this;
	}

	public function current_repository()
	{
		if (isset($this->repository_name) && isset($this->repositories[$this->repository_name])) {
			return $this->repository_name;
		}
		if (count($this->repositories) > 0) {
			return array_shift(array_keys($this->repositories));
		}
		return false;
	}

	public function current_repository_info()
	{
		$name = $this->current_repository();
		return isset($this->repositories[$name]) ? $this->repositories[$name] : array();
	}

	public function get_query($url, $body = null, $method = 'get', $options = array())
	{
		if ($this->connect_error) {
			return false;
		}
		$options = $options + $this->curl_options;
		$agent = Net_HTTP::Agent($options)->act_as_browser();
		$r = Net_HTTP::Request($url)->body($body)->method($method);
		$res = $agent->send($r);
		if ($res->status->code != 200) {
			$this->put_error($url, 'get_query', $res);
			return false;
		}
		$rc = json_decode($res->body);
		if (is_null($rc)) {
			return $res->body;
		}
		return $rc;
	}

	public function get_error()
	{
		return $this->connect_error;
	}

	protected function put_error($url, $method, $res)
	{
		$this->connect_error = array(
			'url' => $url,
			'method' => $method,
			'status' => $res->status->code,
			'response' => $res
		);
		return $this;
	}

	protected function get_composer_content()
	{
		$json = array();
		$main_files = array('../composer.json');
		$components_files = glob(CMS::app_path() . '/components/*/composer.json');
		$files = $components_files ? array_merge($components_files, $main_files) : $main_files;
		foreach ($files as $path) {
			if (IO_FS::exists($path)) {
				$decode = json_decode(IO_FS::File($path)->load(), true);
				if ($decode) {
					$json = array_replace_recursive($json, $decode);
				}
			}
		}
		if (!empty($json)) {
			return json_encode($json, JSON_PRETTY_PRINT);
		}
		return null;
	}

	public function make_item()
	{
		return new CMS_Stockroom_Entity();
	}

	public function create_item($object)
	{
		$item = $this->make_item();
		$item->init_from($object);
		return $item;
	}

	public function load_item($id)
	{
		$o = $this->send('info.json', array('id' => $id));
		if (!$o) {
			return false;
		}
		return $this->create_item($o);
	}

	public function components_count($parms = array())
	{
		$o = $this->send('count.json', $parms);
		if ($this->get_error()) {
			return false;
		}
		if (is_object($o)) {
			if (isset($o->count)) {
				return (int) $o->count;
			}
		}
		return 0;
	}

	public function components_list($parms = array())
	{
		$o = $this->send('select.json', $parms);
		if ($this->get_error()) {
			return false;
		}
		if (is_array($o)) {
			foreach ($o as $n => $item) {
				$o[$n] = $this->create_item($item);
			}
			return $o;
		}
		return array();
	}

	public function get_install_pack($item)
	{
		$tdir = $item->install_temp_dir();
		if (IO_FS::exists($tdir)) {
			CMS::rmdir($tdir);
		}
		CMS::mkdirs($tdir);
		$zip = "{$tdir}/install.zip";
		$res = $item->download();
		if ($res->status->code == 200) {
			$unknown = 'Некорректный инсталляционный пакет!';
			if (isset($res->headers['Content-Type'])) {
				if (Core_Regexps::match('{^application/zip}', $res->headers['Content-Type'])) {
					file_put_contents($zip, $res->body);
					CMS::chmod_file($zip);
					Core::load('IO.Arc');
					IO_Arc::ZIP($zip)->extract_to($tdir);
					unlink($zip);
					return $this->hash_dir($tdir, '', $item);
				} else {
					return $unknown;
				}
			} else {
				return $unknown;
			}
		} else {
			return "Error {$res->status->code} {$res->status->message}";
		}
	}

	protected function hash_dir($path, $prefix = '', $item = false)
	{
		$out = array();
		foreach (IO_FS::Dir($path) as $file) {
			$fp = $prefix == '' ? $file->name : "{$prefix}/{$file->name}";
			if ($fp == 'www') {
				$fp = CMS::www();
			}
			if (is_dir($file->path)) {
				$files = $this->hash_dir($file->path, $fp, $item);
				foreach ($files as $key => $value) {
					$out[$key] = $value;
				}
			} else {
				if ($item && Core_Regexps::match('{\.(php|phtml)$}', $file)) {
					$version = trim($item['version']);
					if ($version != '') {
						$content = file_get_contents($file->path);
						$content = str_replace('%%{version}', $version, $content);
						file_put_contents($file->path, $content);
					}
				}
				$out[$fp] = array(
					'path' => $file->path,
					'hash' => md5_file($file->path),
				);
			}
		}
		return $out;
	}

	public function alter_package_files(&$from, &$to, $item)
	{
		if ($item->is_frontend()) {
			if (preg_match('!\.php$!', $to)) {
				$www = CMS::$www;
				$app = trim(CMS::$app_path, './');
				$to = preg_replace("!/{$www}/!", "/{$app}/", $to, 1);
			}
		}
	}

	public function update_package_files($item, $files = array())
	{
		if (empty($files)) {
			$files = $this->get_install_pack($item);
		}
		if (is_string($files)) {
			$this->last_error = $files;
			return false;
		}
		$updated = array();
		$not_updated = array();
		foreach ($files as $file => $data) {
			$from = $data['path'];
			$to = "../{$file}";
			if ($item->can_update_file($file, $from, $to)) {
				$this->alter_package_files($from, $to, $item);
				if ($m = Core_Regexps::match_with_results('{^(.+)/[^/]+$}', $to)) {
					$dir = $m[1];
					if (!IO_FS::exists($dir)) {
						@CMS::mkdirs($dir);
						if (!IO_FS::exists($dir)) {
							$this->last_error = "Невозможно создать каталог {$dir}";
							return false;
						}
					}
					$new_hash = $data['hash'];
					$old_hash = $item->get_installed_hash($file);
					if (!$old_hash || $old_hash != $new_hash) {
						if ($item->file_was_changed($file)) {
							copy($from, "{$to}.upd");
							CMS::chmod_file("{$to}.upd");
							$not_updated[] = $to;
						} else {
							copy($from, $to);
							CMS::chmod_file($to);
							$updated[] = $to;
						}
						$item->set_installed_hash($file, $new_hash);
					}
				}
			}
		}
		$item->save_info_file();
		CMS::rmdir($item->install_temp_dir());
		$this->update_composer($item);
		return array('updated' => $updated, 'not_updated' => $not_updated);
	}

	public function validate_package_intall($item) {
		foreach ((array) $item->not_install_if_exists as $entry) {
			$entry = $this->validate_path($entry);
			if ($entry) {
				if (IO_FS::exists($entry)) {
					$this->last_error = "<b>Файл (каталог) {$entry}</b> уже существует. Установка невозможна!";
					return false;
				}
			}
		}
		if ($item['download'] == '-') {
			$this->last_error = 'Невозможно получить инсталляционный пакет!';
			return false;
		}
		return true;
	}

	public function validate_path($s)
	{
		$s = trim($s);
		if ($s == '') {
			return false;
		}
		if ($s[0] != '.' && $s[0] != '/') {
			$s = "../{$s}";
		}
		if ($m = Core_Regexps::match_with_results('{^\.\./www/(.+)$}', $s)) {
			$s = '../' . CMS::www() . '/' . $m[1];
		}
		return $s;
	}

	public function install_package_files($item, $files = array())
	{
		if (empty($files)) {
			$files = $this->get_install_pack($item);
		}
		if (is_string($files)) {
			$this->last_error = $files;
			return false;
		}
		foreach ($files as $file => $data) {
			$from = $data['path'];
			$to = "../{$file}";
			$this->alter_package_files($from, $to, $item);
			if ($m = Core_Regexps::match_with_results('{^(.+)/[^/]+$}', $to)) {
				$dir = $m[1];
				if (!IO_FS::exists($dir)) {
					@CMS::mkdirs($dir);
					if (!IO_FS::exists($dir)) {
						$this->last_error = "Невозможно создать каталог {$dir}";
						return false;
					}
				}
				copy($from, $to);
				CMS::chmod_file($to);
				$item->set_installed_hash($file, $data['hash']);
			}
		}
		$item->save_info_file();
		CMS::rmdir($item->install_temp_dir());
		$this->update_composer($item);
		return true;
	}

	public function update_composer($item = null)
	{
		if ($this->is_local_composer()) {
			return $this->update_composer_local($item);
		} else {
			return $this->update_composer_remote($item);
		}
	}

	public function is_local_composer()
	{
		$cmd = WS::env()->config->composer->cmd;
		return !empty($cmd);
	}

	public function update_composer_local($item = null)
	{
		if ($item && !$item->composer_exists()) {
			return false;
		}
		$cmd = WS::env()->config->composer->cmd;
		$content = $this->get_composer_content();
		$dir = '..';
		IO_FS::File($dir . '/composer.json')->update($content);
		$cwd = getcwd();
		chdir($dir);
		exec("$cmd update");
		chdir($cwd);
	}

	public function update_composer_remote($item = null)
	{
		if ($item && !$item->composer_exists()) {
			return false;
		}
		$body = $this->get_composer_content();
		$content = $this->send('composer', array('id' => $item ? $item->id : null), $body, 'post', array(CURLOPT_TIMEOUT => 300));
		if ($content) {
			$filename = '../cache/composer.zip';
			$i = 0;
			while (is_file($filename)) {
				$filename = "../cache/composer{$i}.zip";
			}
			IO_FS::File($filename)->update($content);
			$extract_to = tempnam(sys_get_temp_dir(), 'tao');
			if (file_exists($extract_to)) {
				unlink($extract_to);
			}
			mkdir($extract_to);
			IO_Arc::ZIP($filename)->extract_to($extract_to);
			$this->mirror_dir($extract_to, '..');
			IO_FS::rm($filename);
			print_r($extract_to);
		}
	}

	protected function mirror_dir($source, $dest)
	{
		foreach ($iterator = new \RecursiveIteratorIterator(
					new \RecursiveDirectoryIterator($source, \RecursiveDirectoryIterator::SKIP_DOTS),
					\RecursiveIteratorIterator::SELF_FIRST) as $item
			) {
			$path = $dest . DIRECTORY_SEPARATOR . $iterator->getSubPathName();
			if ($item->isDir()) {
				if (!IO_FS::exists($path)) {
					mkdir($path);
				}
			} else {
				copy($item, $path);
			}
		}
	}

	public function update_core($arc_path = '../tao.zip', $core_path = '../tao', $core_copy_path = '../tao.old')
	{
		$rep = $this->current_repository_info();
		$resp = Net_HTTP::Agent()->to_file($arc_path)->send($rep['core_download_url']);
		if ($resp->status->is_success == 200) {
			Core::load('IO.Arc', 'IO.Arc.ZIP');
			IO_FS::mv($core_path, $core_copy_path);
			IO_Arc::ZIP($arc_path)->extract_to('..');
			IO_FS::rm($arc_path);
			if (is_dir($core_path . '/lib')) {
				IO_FS::chmod_recursive($core_path);
				IO_FS::rm($core_copy_path);
				return true;
			} else {
				IO_FS::mv($core_copy_path, $core_path);
				return 'Error in tao.zip';
			}
		} else {
			return "Error {$resp->status->code}: {$resp->status->message}";
		}
	}

	public function get_core_version()
	{
		$rep = $this->current_repository_info();
		$resp = Net_HTTP::Agent()->send($rep['core_version_url']);
		if ($resp->status->is_success) {
			return json_decode($resp->body);
		}
		return "Error {$resp->status->code}: {$resp->status->message}";
	}
}
