<?php
/**
 * Templates.HTML
 *
 * HTML шаблоны
 *
 * @package Templates\HTML
 * @version 0.2.2
 */

Core::load('Templates', 'Object', 'Cache', 'Text.Insertions', 'Templates.HTML.Assets.Preprocess', 'Templates.HTML.Assets');

/**
 * @package Templates\HTML
 */
class Templates_HTML implements Core_ConfigurableModuleInterface
{
	const VERSION = '0.2.2';

	static protected $helpers;

	static protected $meta;

	static protected $scripts_settings = array();
	static protected $use_files = array();

	protected static $cached_paths = array();

	static protected $options = array(
		'template_class' => 'Templates.HTML.Template',
		'paths' => array(
			'css' => 'styles',
			'less' => 'styles',
			'js' => 'scripts',
			'png' => 'images',
			'jpg' => 'images',
			'jpeg' => 'images',
			'ico' => 'images',
			'gif' => 'images',
			'svg' => 'images',
			'_fallback' => 'files'
		),
		'assets' => array('', '/tao'),
		'copy_dir' => 'copy',
		'extern_file_prefix' => 'file://',
		'css_dir' => 'styles',
		'js_dir' => 'scripts',
		'layouts_path' => 'layouts',
		'join_scripts' => false,
		'join_styles' => false,
		'add_timestamp' => true,
		'absolute_urls' => false,
		'timestamp_pattern' => '%s?%d' // '/nocache/%2$d%1$s'
	);

	/**
	 * Инициализация модуля
	 *
	 * @param array $options
	 */
	static public function initialize(array $options = array())
	{
		self::$helpers = Object::Aggregator()->fallback_to(Templates::helpers());
		foreach (self::$options as $k => $v)
			self::$options[$k] = str_replace('%files%', Core::option('files_name'), $v);
		self::options($options);
		self::use_helper('forms', 'Templates.HTML.Helpers.Forms');
		self::use_helper('tags', 'Templates.HTML.Helpers.Tags');
		self::use_helper('assets', 'Templates.HTML.Helpers.Assets');
		self::use_helper('maps', 'Templates.HTML.Helpers.Maps');
	}

	/**
	 * Устанавливает опции
	 *
	 * @param array $options
	 *
	 * @return mixed
	 */
	static public function options(array $options = array())
	{
		if (count($options)) {
			Core_Arrays::update(self::$options, $options);
		}
		return self::$options;
	}

	/**
	 * Устанавливает/возвращает опцию
	 *
	 * @param string $name
	 * @param        $value
	 *
	 * @return mixed
	 */
	static public function option($name, $value = null)
	{
		$prev = isset(self::$options[$name]) ? self::$options[$name] : null;
		if ($value !== null) {
			self::options(array($name => $value));
		}
		return $prev;
	}

	static public function add_scripts_settings($vars = array(), $replace = false)
	{
		if (!empty($vars)) {
			self::$scripts_settings[] = array($vars, $replace);
		}
		return self::$scripts_settings;
	}

	static public function scripts_settings($vars = array(), $replace = false)
	{
		return self::add_scripts_settings($vars, $replace);
	}

	/**
	 * Регистрирует хелпер
	 *
	 */
	static public function use_helpers()
	{
		$args = Core::normalize_args(func_get_args());
		foreach ($args as $k => $v) {
			self::$helpers->append($v, $k);
		}
	}

	/**
	 * @param string $name
	 * @param        $helper
	 */
	static public function use_helper($name, $helper)
	{
		self::$helpers->append($helper, $name);
	}

	/**
	 * @param string $name
	 */
	static public function helper($name)
	{
		return self::$helpers[$name];
	}

	/**
	 * @param boolean $v
	 */
	static public function join_styles($v = true)
	{
		self::option('join_styles', $v);
	}

	/**
	 * @param boolean $v
	 */
	static public function join_scripts($v = true)
	{
		self::option('join_scripts', $v);
	}

	static public function is_url($path)
	{
		return preg_match('{^(http|https|ftp):}', $path);
	}

	/**
	 * @param string $path
	 * @param bool   $copy
	 *
	 * @return mixed|null|string
	 */
	static public function js_path($path, $copy = true)
	{
		return self::path('js', $path, $copy);
	}

	public static function append_assets_path($path, $position = 1)
	{
		$paths = self::option('assets');
		Core_Arrays::put($paths, $path, $position);
		$paths = array_unique($paths);
		self::option('assets', $paths);
		return $paths;
	}

	/**
	 * @param string $path
	 * @param bool   $copy
	 * @return mixed|null|string
	 */
	static public function css_path($path, $copy = true)
	{
		return self::path('css', $path, $copy);
	}

	public function reset_paths_cache()
	{
		self::$cached_paths = array();
	}

	public static function path($type, $path, $copy = true, $link = false)
	{
		$rc = Events::call('templates.asset.path', $type, $path, $copy, $link);
		if (!is_null($rc)) {
			return $rc;
		}
		$key = "$type;$path;$copy;$link";
		if (isset(self::$cached_paths[$key])) {
			return self::$cached_paths[$key];
		}
		$path = self::extern_filepath($path, $copy, $link);
		if (Templates::is_absolute_path($path) || self::is_url($path)) {
			return self::$cached_paths[$key] = $path;
		}
		$urls = array();
		foreach (self::option('assets') as $asset) {
			if ($asset[0] == '.') {
				$asset = self::option('extern_file_prefix') . $asset;
			}
			$url = sprintf("%s/%s/%s", $asset, self::option("{$type}_dir"), $path);
			$url = self::extern_filepath($url, $copy, $link);
			$urls[] = $url;
			$file = ".{$url}";
			if (is_file($file)) {
				return self::$cached_paths[$key] = $url;
			}
		}
		if (Core::option('deprecated')) {
			$file = Core::tao_deprecated_file('files/' . self::option("{$type}_dir") . '/' . $path);
			if (is_file($file)) {
				return self::$cached_paths[$key] = self::extern_filepath(self::option('extern_file_prefix') . $file, $copy, $link);
			}
		}
		return self::$cached_paths[$key] = null;
	}

	public static function file_to_docroot($file, $dest = null, $copy_dir = '', $link = false)
	{
		return self::extern_filepath($file, true, $link, $dest, $copy_dir);
	}

	/**
	 * @param      $file
	 * @param bool $copy
	 * @param bool $link
	 * @param null $dest
	 * @param null $copy_dir
	 * @return mixed|string
	 */
	static public function extern_filepath($file, $copy = true, $link = false, $dest = null, $copy_dir = null)
	{
		$file = (string)$file;
		$prefix = self::option('extern_file_prefix');
		if (Core_Strings::starts_with($file, $prefix)) {
			$res = str_replace($prefix, '', $file);
			if (is_null($dest)) {
				$res_name = ltrim(str_replace(Core::tao_dir(), '', $res), '/.');
			} else {
				$res_name = rtrim($dest, '/.') . '/' . basename($file);
			}
			if ($copy == false) {
				return $res;
			}
			$ext = strtolower(pathinfo($file, PATHINFO_EXTENSION));
			$paths = Templates_HTML::option('paths');
			$mtime = @filemtime($res);
			if ($mtime) {
				$dir = trim(is_null($copy_dir) ? self::option('copy_dir') : $copy_dir, '/');
				$root = $paths['_fallback'];
				if (isset($paths[$ext])) {
					$root = $paths[$ext];
				}
				$base_dir = trim($root . '/' . $dir, '/');
				$path = sprintf('%s/%s', $base_dir, $res_name);
				$dir = dirname($path);
				if (!is_dir($dir)) {
					IO_FS::mkdir($dir);
				}
				$url = '/' . ltrim($path, '.');
				if ($link) {
					if (!is_link($path)) {
						if (is_file($path)) {
							IO_FS::rm($path);
						}
						symlink(realpath($res), $path);
					} else {
						$link_path = readlink($path);
						if (!is_file($link_path) || realpath($link_path) != realpath($res)) {
							unlink($path);
							symlink(realpath($res), $path);
						}
					}
					return $url;
				} else {
					$mtime_exists = @filemtime($path);
					if (($mtime > $mtime_exists && IO_FS::cp($res, $path)) || IO_FS::exists($path)) {
						return $url;
					}
				}
			} else {
				return $res;
			}
		}
		return $file;
	}

	/**
	 * Возвращает делигатор хелперов
	 *
	 * @return Object_Aggregator
	 */
	static public function helpers()
	{
		return self::$helpers;
	}

	/**
	 * Фабричный метод, возвращает объект класса Templates.HTML.Template (или его наследника)
	 *
	 * @param string $name
	 * @param array  $parameters
	 * @param null   $js_aggregator
	 * @param null   $css_aggregator
	 *
	 * @return CMS_Views_View|Templates_Template
	 */
	static public function Template($name, array $parameters = array(), $js_aggregator = null, $css_aggregator = null)
	{
		return Core::make(self::$options['template_class'], $name, $parameters, $js_aggregator, $css_aggregator);
	}

	/**
	 * Метод, который устанавливает статическое свойство meta(если передан параметр) и возвращает значение этого свойства
	 *
	 * @param Templates_HTML_Metas $meta
	 *
	 * @return Templates_HTML_Metas
	 */
	static public function meta(Templates_HTML_Metas $meta = null)
	{
		if ($meta) {
			self::$meta = $meta;
			WS::env()->meta = $meta;
		} else if (!is_object(self::$meta)) {
			self::$meta = new Templates_HTML_Metas();
		}
		return self::$meta;
	}

	public static function use_file($name, $parms = array())
	{
		$parms['name'] = $name;
		self::$use_files[$name] = $parms;
	}

	public static function get_use_files()
	{
		return self::$use_files;
	}



//TODO: deprecated
	/*
	  static public function views_path_for($path) {
		return Templates::get_path($path);
	  }

	  static public function exists($name) {
		return file_exists(self::views_path_for(self::file_name_for($name)));
	  }

	  static public function file_name_for($name) {
		return $name.(preg_match('{\.[a-zA-Z0-9]+$}', (string) $name) ? '' : '.phtml');
	  }
	*/

}

/**
 * Класс HTML шаблона
 *
 * @package Templates\HTML
 *
 * @property Templates_HTML_Metas $meta
 */
class Templates_HTML_Template
	extends Templates_NestableTemplate
	implements Core_IndexedAccessInterface
{

	protected $content;
	protected $extension = '.phtml';
	/** @var Templates_HTML_Assets_Agregator[] $agregators */
	protected $agregators = array('css' => null, 'js' => null);
	protected $include_meta = true;
	protected $is_layout = false;
	protected $allow_filtering = true;
	protected $spawn_from;
	protected $scripts_settings = array();
	protected $enable_less = true;
	protected $enable_scss = true;
	protected $no_duplicates = array();
	protected $partial_paths = array();
	protected $use_onpage_to_file = array('js' => false, 'css' => false);
	protected $default_use_attributes = array('js' => false, 'css' => false);
	protected $head_parts = array('metas', 'inline', 'files_css', 'files_js', 'inline_js', 'inline_css');
	protected $frontend;

	/**
	 * Конструктор
	 *
	 * @param string $name
	 */
	public function __construct($name, array $parameters = array(), $js_agregator = null, $css_agregator = null)
	{

		$this->setup_agregators($js_agregator, $css_agregator);
		parent::__construct($name);
		$this->content = new ArrayObject();
		$this->with($parameters);
	}

	protected function setup_agregators($js_agregator = null, $css_agregator = null)
	{
		$this->agregators['js'] = ($js_agregator instanceof Templates_HTML_Assets_AgregatorInterface) ?
			$js_agregator : Templates_HTML_Assets::agregator(array(), 'js');
		$this->agregators['css'] = ($css_agregator instanceof Templates_HTML_Assets_AgregatorInterface) ?
			$css_agregator : Templates_HTML_Assets::agregator(array(), 'css');
	}

	protected function setup($js_agregator = null, $css_agregator = null)
	{
		if ($this->enable_less) {
			$this->enable_less();
		}
		if ($this->enable_scss) {
			$this->enable_scss();
		}
		$this->join_scripts(Templates_HTML::option('join_scripts'));
		$this->join_styles(Templates_HTML::option('join_styles'));
	}

	public function clear($name)
	{
		if ($this->record) {
			$this->record_call(__FUNCTION__, func_get_args());
		}
		$this->content[$name] = '';
		return $this;
	}

	public function clear_all()
	{
		foreach ($this->content as $k => $v) {
			$this->clear($k);
		}
		return $this;
	}

	public function no_duplicates_in($name)
	{
		$this->no_duplicates[$name] = true;
		return $this;
	}

	public function merge($obj)
	{
		parent::merge($obj);
		$this->include_meta = $obj->include_meta;
		$this->allow_filtering($obj->allow_filtering);
		return $this;
	}

	public function use_attrs($type, $attrs = array())
	{
		$this->default_use_attributes[$type] = $attrs;
		return $this;
	}

	public function enable_scss()
	{
		if (!$this->agregators['css']->get_preprocessor('scss')) {
			$this->agregators['css']->add_preprocessor('scss', 'Templates.HTML.Assets.Preprocess.SCSS');
		}
		return $this;
	}

	public function disable_scss()
	{
		$this->agregators['css']->remove_preprocessor('scss');
		return $this;
	}

	public function enable_less()
	{
		if (!$this->agregators['css']->get_preprocessor('less')) {
			$this->agregators['css']->add_preprocessor('less', 'Templates.HTML.Assets.Preprocess.LESS');
		}
		return $this;
	}

	public function disable_less()
	{
		$this->agregators['css']->remove_preprocessor('less');
		return $this;
	}

	public function minify_styles($v = true)
	{
		if ($v) {
			Core::load('Templates.HTML.Assets.Postprocess');
			$this->agregators['css']->add_postprocessor('minify', Templates_HTML_Assets_Postprocess::MinifyCSS());
		} else {
			$this->agregators['css']->remove_postprocessor('minify');
		}
		return $this;
	}

	public function minify_scripts($v = true)
	{
		if ($v) {
			Core::load('Templates.HTML.Assets.Postprocess');
			$this->agregators['js']->add_postprocessor('minify', Templates_HTML_Assets_Postprocess::MinifyJS());
		} else {
			$this->agregators['js']->remove_postprocessor('minify');
		}
		return $this;
	}

	public function allow_filtering($v = true)
	{
		$this->allow_filtering = $v;
		return $this;
	}

	public function join_styles_group()
	{
		$args = func_get_args();
		call_user_func_array(array($this->agregators['css'], 'join_group'), $args);
		return $this;
	}

	public function join_scripts_group()
	{
		$args = func_get_args();
		call_user_func_array(array($this->agregators['js'], 'join_group'), $args);
		return $this;
	}

	public function join_styles($v = true)
	{
		$this->agregators['css']->join($v);
		return $this;
	}

	public function join_scripts($v = true)
	{
		$this->agregators['js']->join($v);
		return $this;
	}

	public function no_join()
	{
		foreach (Core::normalize_args(func_get_args()) as $file)
			$this->agregators[pathinfo($file, PATHINFO_EXTENSION)]->exclude_join($file);
		return $this;
	}

	/** @deprecated */
	public function frontend_css($name, $attrs = array())
	{
		return $this->frontend()->css_tag($name, $attrs);
	}

	/** @deprecated */
	public function frontend_js($name, $attrs = array())
	{
		return $this->frontend()->js_tag($name, $attrs);
	}

	/** @deprecated */
	public function frontend_url($path)
	{
		return $this->frontend()->url($path);
	}

	public function frontend()
	{
		return $this->frontend ?: $this->frontend = $this->setup_frontend();
	}

	public function use_file(array $file, $type = null)
	{
		if ($this->record) {
			$this->record_call(__FUNCTION__, func_get_args());
		}
		if (!Templates_HTML::is_url($file['name'])) {
			$ext = pathinfo($file['name'], PATHINFO_EXTENSION);
			if (empty($ext) && !empty($type) && !Templates::is_absolute_path($file['name'])) {
				$ext = $type;
				$file['name'] = $file['name'] . '.' . $ext;
			}
			if ($type === null) {
				$type = $ext;
			}
		}
		if (is_array($this->default_use_attributes[$type])) {
			$file = array_merge($this->default_use_attributes[$type], $file);
		}
		if (isset($this->agregators[$type])) {
			$this->agregators[$type][$file['name']] = $file;
		}
		return $this;
	}

	protected function parse_package_args($args)
	{
		$packs = array();
		if (count($args) == 1 && is_array($args[0])) {
			$packs = $args[0];
		} else {
			while ($value = array_shift($args)) {
				if (is_string($value)) {
					if (is_array(reset($args))) {
						$packs[$value] = array_shift($args);
					} else {
						$packs[$value] = array();
					}
				}
			}
		}
		return $packs;
	}

	public function use_package()
	{
		$args = func_get_args();
		$packs = $this->parse_package_args($args);
		Core::load('CMS.Packages');
		foreach ($packs as $name => $config) {
			$pack = CMS_Packages::find($name);
			if (!$pack) {
				$pack = CMS_Packages::auto_install($name, $config);
			}
			if ($pack) {
				$pack->use_in_template($this, $config);
			}

		}
		return $this;
	}

	public function use_script($file, array $parms = array())
	{
		$parms['name'] = $file;
		return $this->use_file($parms, 'js');
	}

	public function use_frontend_script($entry, $params = array())
	{
		return $this->use_script($this->frontend()->js_url($entry), $params);
	}

	public function use_style($file, array $parms = array())
	{
		$parms['name'] = $file;
		return $this->use_file($parms, 'css');
	}

	public function use_frontend_style($entry, $params = array())
	{
		if ($this->frontend()->is_hot()) {
			return '';
		}
		return $this->use_style($this->frontend()->css_url($entry), $params);
	}

	public function use_styles()
	{
		foreach (Core::normalize_args(func_get_args()) as $file)
			$this->use_file(is_array($file) ? $file : array('name' => $file), 'css');
		return $this;
	}

	public function use_scripts()
	{
		foreach (Core::normalize_args(func_get_args()) as $file)
			$this->use_file(is_array($file) ? $file : array('name' => $file), 'js');
		return $this;
	}

	public function add_scripts_settings($vars = array(), $replace = false)
	{
		if (!empty($vars)) {
			if ($this->record) {
				$this->record_call(__FUNCTION__, func_get_args());
			}
			$this->root->scripts_settings[] = array($vars, $replace);
		}
		return $this;
	}

	public function response_data()
	{
		$this->root->build_head();
		return array(
			'data' => $this->render(),
			'eval' => trim($this->root['js']),
			'js' => $this->js_agregator->files_list(),
			'css' => $this->css_agregator->files_list()
		);
	}

	public function get_scripts_settings()
	{
		return array_merge(Templates_HTML::scripts_settings(), $this->scripts_settings);
	}

	protected function build_scripts_settings()
	{
		$settings = $this->get_scripts_settings();
		if (!empty($settings)) {
			$values = array();
			foreach ($settings as $val)
				if ($val[1]) {
					$values = Core_Arrays::deep_merge_update($values, $val[0]);
				} else {
					$values = array_merge_recursive($values, $val[0]);
				}
			$src = 'jQuery.extend(true, TAO.settings, ' . json_encode($values) . ');';
			$this['js'] = $src . $this['js'];
		}
		return '';
	}

	public function use_onpage_to_file($type = 'js', $v = true)
	{
		$this->use_onpage_to_file[$type] = $v;
		return $this;
	}

	protected function onpage_to_file($type = 'js')
	{
		$enable = $this->use_onpage_to_file[$type];
		if ($enable) {
			$content = $this[$type];
			$request = WS::env()->request;
			$path = 'onpage' . '/' . trim($request->path, '/.');
			$method = "{$type}_path";
			$dir = '.' . Templates_HTML::$method($path);
			$name = md5($content);
			// $name = md5($request->query);
			$file = $dir . '/' . $name . '.' . $type;
			if (!is_file($file)) {
				IO_FS::rm($dir);
				IO_FS::mkdir($dir);
				file_put_contents($file, $content);
			}
			$this->use_file(array('name' => trim($file, '.'), 'weight' => 100000));
			$this[$type] = '';
		}
		return $this;
	}

	public function include_files($place = null, $type = null)
	{
		if (empty($place)) {
			$place = null;
		}
		$res = '';
		if (is_null($type) || $type == 'js') {
			$this->onpage_to_file('js');
		}
		if (is_null($type) || $type == 'css') {
			$this->onpage_to_file('css');
		}

		foreach (Templates_HTML::get_use_files() as $name => $parms) {
			$this->use_file($parms);
		}

		if (is_null($type) || $type == 'js') {
			$settings = $this->get_scripts_settings();
			if (!empty($settings)) {
				$this->use_script('/tao/scripts/tao.js', array('type' => 'lib', 'weight' => -15));
			}
		}
		$agregators = is_null($type) ? $this->agregators : (isset($this->agregators[$type]) ? array($type => $this->agregators[$type]) : array());

		foreach ($agregators as $type => $a) {
			$method = $type . '_link';
			foreach ($a->files_list($place) as $file_path => $file) {
				$options = !empty($file) && isset($file['attrs']) ? $file['attrs'] : array();
				$add_timestamp = !empty($file) && isset($file['add_timestamp']) ? $file['add_timestamp'] : Templates_HTML::option('add_timestamp');
				$absolute = !empty($file) && isset($file['absolute']) ? $file['absolute'] : null;
				if (method_exists($this, $method)) {
					$res .= $this->$method($file_path, $options, $add_timestamp, $absolute);
				} else {
					$res .= $this->get_helpers()->$method($file_path, $options, $add_timestamp, $absolute);
				}
			}
		}
		return $res;
	}

	protected function configure_layoute()
	{
		$this
			->use_attrs('js', array('type' => 'lib'))
			->use_attrs('css', array('type' => 'lib'));
		$env = Config::all()->environment;
		if ($env == 'prod' && !Core_Strings::contains($this->name, 'admin')) {
			$this
				->use_onpage_to_file('js')
				->use_onpage_to_file('css')
				->no_duplicates_in('js')
				->join_styles()->minify_styles()
				->join_scripts()->minify_scripts();
		}
		return $this;
	}

	public function within_layout($layout)
	{
		if (!Templates::is_absolute_path($layout)) {
			$layout = Templates_HTML::option('layouts_path') . '/' . $layout;
		}
		return $this->inside(Templates_HTML::Template($layout)->is_layout(true));
	}

	public function is_layout($v = null)
	{
		if (is_null($v)) {
			return $this->is_layout;
		} else {
			$this->is_layout = $v;
			return $this;
		}
	}

	public function no_layout()
	{
		if ($this->record) {
			$this->record_call(__FUNCTION__, func_get_args());
		}
		return $this->pull();
	}

	public function with_parameters(array $parameters = array())
	{
		return $this->with($parameters);
	}

	/**
	 * Устанавливает внутри какого шаблона находиться данный шаблон
	 *
	 * @param Templates_Text_Template $container
	 *
	 * @return Templates_NestableTemplate
	 */
	public function inside(Templates_NestableTemplate $container)
	{
		parent::inside($container);
		foreach ($this->agregators as $name => $ag) {
			foreach ($ag as $fname => $finfo) {
				if (!isset($container->agregators[$name][$fname])) {
					$container->agregators[$name]->add_file_array($fname, $finfo);
				}
			}
		}
		$this->agregators = $container->agregators;
		return $this;
	}

	public function copy($name, $with_content = true)
	{
		$res = clone $this;
		$res->no_layout();
		$res->name = $name;
		if (!$with_content) {
			$res->content = Core::hash();
		}
		return $res;
	}

	public function spawn($name)
	{
		$res = $this->copy($name, false);
		$res->spawn_from = $this->root;
		return $res;
	}

	/**
	 * Начинает запись блока
	 *
	 * @param  $block
	 *
	 * @return Templates_NestableTemplate
	 */
	public function begin($block)
	{
		ob_start();
		return $this;
	}

	/**
	 * Заканчивает запись блока и сохраняет его
	 *
	 * @param $block
	 * @param bool $append добавить или заменить текущий блок $prepend
	 *
	 * @return Templates_NestableTemplate
	 */
	public function end($block, $append = true)
	{
		$value = ob_get_clean();
		return $this->content($block, $value, $append);
		// $this->content[$block] = ($append && isset($this->content[$block]) ? $this->content[$block] : '') . $value;
		// return $this;
	}

	/**
	 * Создает блок и заполняет его контентом
	 *
	 * @param string  $name
	 * @param string  $content
	 * @param boolean $prepend
	 *
	 * @return Templates_HTML_Template
	 */
	public function content($name, $content, $prepend = true)
	{
		if ($this->record) {
			$this->record_call(__FUNCTION__, func_get_args());
		}
		$old_content = isset($this->content[$name]) ? $this->content[$name] : '';

		$root = $this->root;
		$spawn = $this->spawn_from;
		if ($this->no_duplicates[$name] &&
			($spawn && Core_Strings::contains($spawn[$name], $content)) || (Core_Strings::contains($this[$name], $content) || Core_Strings::contains($root[$name], $content))
		) {
			return $this;
		}

		if ($prepend) {
			$this->content[$name] = $content . $old_content;
		} else {
			$this->content[$name] = $old_content . $content;
		}
		return $this;
	}

	/**
	 * Добавить контент к концу блоку
	 *
	 * @param string $name
	 * @param string $content
	 *
	 * @return Templates_HTML_Template
	 */
	public function prepend_to($name, $content)
	{
		return $this->content($name, $content, true);
	}

	/**
	 * Добавить контент к началу блоку
	 *
	 * @param string $name
	 * @param string $content
	 *
	 * @return Templates_HTML_Template
	 */
	public function append_to($name, $content)
	{
		return $this->content($name, $content, false);
	}

	/**
	 * Добавляет контент к блоку, если этот блок пустой
	 *
	 * @param string $block
	 * @param string $content
	 *
	 * @return Templates_HTML_Template
	 */
	public function if_empty($block, $content)
	{
		if (!isset($this->content[$block])) {
			$this->content($block, $content);
		}
		return $this;
	}

//TODO: deprecated use meta
//-----------------------
	/**
	 * Зополняет блок title, если он пустой
	 *
	 * @param string $content
	 *
	 * @return Templates_HTML_Template
	 */
	public function title($content)
	{
		return $this->if_empty('title', htmlspecialchars($content));
	}

	/**
	 * Зополняет блок description, если он пустой
	 *
	 * @param string $content
	 *
	 * @return Templates_HTML_Template
	 */
	public function description($content)
	{
		return $this->if_empty('description', htmlspecialchars($content));
	}

//-----------------------

	protected function partial_cache_key($name, $parms)
	{
		return $this->cache_key($name, $parms, 'partial');
	}

	/**
	 * @param string $name    Имя шаблона
	 * @param array  $params  Параметры для шаблона
	 * @param null   $key     Ключ для хранилища кеша
	 * @param null   $timeout Время жизни кеша
	 * @return string
	 */
	public function partial_cache($name, $params = array(), $key = null, $timeout = null)
	{
		$key = $key ? $key : $this->partial_cache_key($name, $params);
		return $this->cached_call($key, array($this, 'partial'), array($name, $params), $timeout);
	}

	/**
	 * Возвращает результат шаблона с именем $__name
	 *
	 * @param       $__name
	 * @param array $__params
	 *
	 * @return string
	 * @throws Templates_MissingTemplateException
	 */
	public function partial($__name, $__params = array())
	{
		Events::call('templates.partial', $this, $__name, $__params);
		extract(array_merge($this->get_parms(), $__params));

		if (IO_FS::exists($__path = $this->get_partial_path($__name))) {
			ob_start();
			include($__path);
			$__content = ob_get_clean();
			//return $__content;
			return $this->filter($__content);
		} else {
			throw new Templates_MissingTemplateException($__path);
		}
	}

	/**
	 * Компанует вместе несколько вызовов partial или/и строки
	 *
	 * @return string
	 */
	public function compose()
	{
		$r = '';
		$args = func_get_args();
		foreach ($args as $part) {
			$r .= is_array($part) ?
				call_user_func_array(array($this, 'partial'), $part) :
				(string)$part;
		}
		return $r;
	}

	/**
	 * Формирует html-таг
	 *
	 * @param string  $name
	 * @param array   $attrs
	 * @param boolean $close
	 *
	 * @return string
	 */
	public function tag($name, array $attrs = array(), $close = true)
	{
		$tag = '<' . ((string)$name);

		foreach ($attrs as $k => $v)
			$tag .= ($v === true ? " $k " : ($v === false ? '' : " $k=\"" . htmlspecialchars((string)$v) . '"'));

		return $tag .= (boolean)$close ? ' />' : '>';
	}

	/**
	 * Формирует таг с контеном
	 *
	 * @param string $name
	 * @param string $content
	 * @param array  $attrs
	 *
	 * @return string
	 */
	public function content_tag($name, $content, array $attrs = array())
	{
		$tag = '<' . ((string)$name);

		foreach ($attrs as $k => $v) {
			$tag .= ($v === true ? " $k " : ($v === false ? '' : " $k=\"" . htmlspecialchars($v) . '"'));
		}

		return $tag .= '>' . ((string)$content) . '</' . ((string)$name . '>');
	}

	public function attributes($attrs)
	{
		$tag = '';
		foreach ($attrs as $k => $v) {
			$tag .= ($v === true ? " $k " : ($v === false ? '' : " $k=\"" . htmlspecialchars($v) . '"'));
		}
		return $tag;
	}

	/**
	 * Формирует таг a
	 *
	 * @param string $url
	 * @param string $content
	 * @param array  $attrs
	 *
	 * @return string
	 */
	public function link_to($url, $content, array $attrs = array())
	{
		return $this->content_tag('a', $content, array_merge($attrs, array('href' => $url)));
	}

	/**
	 * Формирует mailto ссылку
	 *
	 * @param string $address
	 * @param string $body
	 * @param array  $attributes
	 *
	 * @return string
	 */
	public function mail_to($address, $body = '', array $attributes = array())
	{
		return $this->link_to("mailto:$address", ($body ? $body : $address), $attributes);
	}

	/**
	 * Формирует html-форму с кнопкой, отправляющей запрос по указанному адресу
	 *
	 * @param string $url
	 * @param string $text
	 * @param string $method
	 * @param array  $attributes
	 *
	 * @return string
	 */
	public function button_to($url, $text, $method = 'get', array $attributes = array())
	{
		$confirmation = Core_Arrays::pick($attributes, 'confirm');
		return $this->
			content_tag('form',
				(($method == 'get' || $method == 'post') ? '' :
					$this->tag('input', array(
							'type' => 'hidden', 'name' => '_method', 'value' => $method)
					)) .
				$this->content_tag('button', $text, array(
						'type' => 'submit',
						'onclick' => $confirmation ? 'return ' . $this->make_confirmation($confirmation) . ';' : false,
					) + $attributes
				),
				array(
					'action' => $url,
					'method' => ($method == 'get' || $method == 'post' ? $method : 'post'))
			);
	}

	/**
	 * Формирует кнопку, отправляющую запрос по указанному адресу
	 *
	 * @param string $url
	 * @param string $text
	 * @param string $method
	 * @param array  $attributes
	 *
	 * @return string
	 */
	public function form_button_to($url, $text, $method = 'get', array $attributes = array())
	{
		$confirmation = Core_Arrays::pick($attributes, 'confirm');
		return $this->tag('input', array(
				'value' => $text,
				'type' => 'submit',
				'onclick' => "this.form.action='$url';" .
					'this.form.method=\'' . ($method == 'get' || $method == 'post' ? $method : 'post') . '\';' .
					"this.form.elements._method.value='$method';" .
					'return ' . $this->make_confirmation($confirmation) . ';') +
			$attributes
		);
	}

	/**
	 * Формирует img таг
	 *
	 * @param string $url
	 * @param array  $attrs
	 *
	 * @return string
	 */
	public function image($url, array $attrs = array())
	{
		return $this->tag('img', array_merge($attrs, array('src' => $url)));
	}

	/**
	 * Формирует таг script ссылающийся на указанный js-скрипт
	 *
	 * @param string $path
	 * @param array  $options
	 *
	 * @return string
	 */
	public function js_link($path, array $options = array(), $add_timestamp = true, $absolute = null)
	{
		return $this->content_tag('script', '',
			array_merge(array(
					'src' => $this->js_path($path, $add_timestamp, $absolute)
				),
				$options
			)
		) . "\n";
	}

	/**
	 * Формирует link таг ссылающийся на указанный css-файл
	 *
	 * @param string $path
	 * @param array  $options
	 *
	 * @return string
	 */
	public function css_link($path, array $options = array(), $add_timestamp = true, $absolute = null)
	{
		return $this->tag('link',
			array_merge(array(
					'rel' => 'stylesheet',
					'href' => $this->css_path($path, $add_timestamp, $absolute)
				),
				$options
			)
		) . "\n";
	}

	protected function js_path($path, $add_timestamp = true, $absolute = null)
	{
		$path = Templates_HTML::js_path($path);
		$url = $this->create_asset_url($path, $absolute);
		return $add_timestamp ? $this->add_timestamp($path, $url) : $url;
	}

	protected function create_asset_url($path, $absolute = null)
	{
		if ((
				(is_null($absolute) && Templates_HTML::option('absolute_urls'))
				|| $absolute
			)
			&& !Templates_HTML::is_url($path)
		) {
			return WS::env()->request->path($path)->query('')->url;
		}
		return $path;
	}

	protected function add_timestamp($path, $link)
	{
		$ts = @filemtime('./' . ltrim($path, '/'));
		return !empty($ts) ?
			sprintf(Templates_HTML::option('timestamp_pattern'), $link, $ts) :
			$link;
	}

	protected function css_path($path, $add_timestamp = true, $absolute = null)
	{
		$path = Templates_HTML::css_path($path);
		$url = $this->create_asset_url($path, $absolute);
		return $add_timestamp ? $this->add_timestamp($path, $url) : $url;
	}

	/**
	 * @return Templates_HTML_Template
	 */
	public function js($string)
	{
		if (empty($string)) {
			return '';
		}
		return $this->content_tag('script', $string);
	}

	/**
	 * @return Templates_HTML_Template
	 */
	public function css($string)
	{
		if (empty($string)) {
			return '';
		}
		return $this->content_tag('style', $string, array('type' => 'text/css'));
	}

	/**
	 * Доступ на чтение к свойствам объекта
	 *
	 * @param string $property
	 *
	 * @return mixed
	 * @throws Core_MissingPropertyException
	 */
	public function __get($property)
	{
		switch (true) {
			case $property == 'meta':
				return Templates_HTML::meta();
			case $property == 'no_duplicates':
				if (!empty($this->$property)) {
					return $this->$property;
				}
				if (!$this->is_root && !empty($this->root->$property)) {
					return $this->root->$property;
				}
				// if ($this->spawn_from) var_dump($this->spawn_from->name);
				if ($this->spawn_from) {
					return $this->spawn_from->$property;
				}
				return null;
			case in_array($property, array('agregators', 'include_meta', 'allow_filtering', 'content', 'scripts_settings')):
				return $this->$property;
			case Core_Strings::ends_with($property, '_agregator'):
				$names = explode('_', $property);
				if (isset($this->agregators[$names[0]])) {
					return $this->agregators[$names[0]];
				} else {
					throw new Core_MissingPropertyException($property);
				}
			default:
				return parent::__get($property);
		}
	}

	/**
	 * Доступ на запись к свойствам объекта
	 *
	 * @param string $property
	 * @param        $value
	 *
	 * @return mixed
	 * @throws Core_MissingPropertyException
	 * @throws Core_ReadOnlyPropertyException
	 */
	public function __set($property, $value)
	{
		switch (true) {
			case in_array($property, array('agregators', 'meta')):
				throw new Core_ReadOnlyPropertyException($property);
			case in_array($property, array('include_meta', 'allow_filtering')):
				$this->$property = (boolean)$value;
				return $this;
			case Core_Strings::ends_with($property, '_agregator'):
				$names = explode('_', $property);
				if ($value instanceof Templates_HTML_FilesAgregatorInterface) {
					$this->agregators[$names[0]] = $value;
					return $this;
				} else {
					throw new Core_MissingPropertyException($property);
				}
			default:
				return parent::__set($property, $value);
		}
	}

	/**
	 * Проверяет установленно ли свойство объекта
	 *
	 * @param string $property
	 *
	 * @return boolean
	 */
	public function __isset($property)
	{
		switch (true) {
			case in_array($property, array('agregators', 'include_meta', 'meta', 'allow_filtering', 'no_duplicates')):
				return true;
			case Core_Strings::ends_with($property, '_agregator'):
				$names = explode('_', $property);
				return isset($this->agregators[$names[0]]);
			default:
				return parent::__isset($property);
		}
	}

	/**
	 * Очищает свойство объекта
	 *
	 * @param string $property
	 *
	 * @return $this|void
	 * @throws Core_UndestroyablePropertyException
	 */
	public function __unset($property)
	{
		switch (true) {
			case in_array($property, array('agregators', 'include_meta', 'meta', 'allow_filtering')):
				throw new Core_UndestroyablePropertyException($property);
			case Core_Strings::ends_with($property, '_agregator'):
				$names = explode('_', $property);
				unset($this->agregators[$names[0]]);
				return $this;
			default:
				return parent::__unset($property);
		}
	}

	public function __sleep()
	{
		return array_merge(parent::__sleep(),
			array('include_meta', 'allow_filtering', 'enable_less', 'views_paths', 'name', 'agregators')
		);
	}

	public function build_head_insertion($arg_str = '', $args = array())
	{
		unset($args['modifer']);
		return call_user_func_array(array($this, 'build_head'), $args);
	}

	public function build_head()
	{
		$res = '';
		/**
		 * @event templates.build_head
		 * Вызывается перед формированием заголовочной части шаблона
		 */
		Events::call('templates.build_head', $this);
		foreach ($this->parse_head_parts_args(func_get_args()) as $part) {
			$method_name = 'build_head_' . $part;
			if (method_exists($this, $method_name)) {
				$res .= $this->$method_name();
			}
		}
		return $res;
	}

	public function build_head_metas()
	{
		return $this->include_meta ? Templates_HTML::meta() : '';
	}

	public function build_head_inline()
	{
		return isset($this->content['head']) ? $this->content['head'] : '';
	}

	public function build_head_inline_js()
	{
		$this->build_scripts_settings();
		return $this->js($this['js']) . "\n";
	}

	public function build_head_inline_css()
	{
		return $this->css($this['css']) . "\n";
	}

	public function build_head_files()
	{
		return $this->include_files();
	}

	public function build_head_files_js()
	{
		return $this->include_files(null, 'js');
	}

	public function build_head_files_css()
	{
		return $this->include_files(null, 'css');
	}

	public function parse_head_parts_args($args)
	{
		$head_parts = $this->head_parts;
		if($args) {
			$head_parts = array();
			if (Core_Strings::starts_with($args[0], '^')) {
				$exact_mode = true;
				$args[0] = ltrim($args[0], '^');
			}

			foreach ($this->head_parts as $part) {
				if ($exact_mode XOR in_array($part, $args)) {
					$head_parts[] = $part;
				}
			}
		}
		return $head_parts;
	}

	public function head_line($line)
	{
		$this->append_to('head', $line . "\n");
		return $this;
	}

	/**
	 * Возвращает содержтмое блока с именем $index
	 *
	 * @param string $index
	 *
	 * @return mixed
	 */
	public function offsetGet($index)
	{
		switch (true) {
			case $index == 'include_files':
				return $this->include_files();
			case $index == 'head':
				return '%head{}';
			default:
				return isset($this->content[$index]) ? $this->content[$index] : '';
		}
		// isset($this->content[$index]) ? $this->content[$index] : '';
	}

	/**
	 * Устанавливает содержимое блока
	 *
	 * @param string $index
	 * @param        $value
	 *
	 * @return mixed
	 */
	public function offsetSet($index, $value)
	{
		return $this->clear($index)->content($index, (string)$value);
	}

	/**
	 * Проверяет существует ли блок
	 *
	 * @param string $index
	 *
	 * @return boolean
	 */
	public function offsetExists($index)
	{
		return isset($this->content[$index]);
	}

	/**
	 * Удаляет блок
	 *
	 * @param string $index
	 */
	public function offsetUnset($index)
	{
		unset($this->content[$index]);
	}

	/**
	 * Формирует js confirm вызов
	 *
	 * @param string $text
	 *
	 * @return string
	 */
	protected function make_confirmation($text)
	{
		return $text ? "confirm('" . Core_Strings::replace($text, "'", "\'") . "')" : 'true';
	}

	public function set_patrial_paths($paths = array(), $base_name = '')
	{
		$suffix = trim(preg_replace('{/?[^/]+$}', '', $base_name), './');
		if ($base_name && $suffix) {
			foreach ($paths as $k => $p) {
				$paths[$k] = rtrim($p, '/') . '/' . $suffix;
			}
		}
		$this->partial_paths = $paths;
		return $this;
	}

	public function partial_paths($paths = array(), $base_name = '')
	{
		if (!empty($paths)) {
			$this->set_patrial_paths($paths, $base_name);
		}
		$result = $this->partial_paths;
		$base_path = rtrim(preg_replace('{/[^/]+$}', '', $this->get_path()), '/');
		if (array_search($base_path, $result) === false) {
			$result[] = $base_path;
		}
		return $result;
	}

	/**
	 * Возвращает путь до partial шаблона
	 *
	 * @param string $name
	 *
	 * @return string
	 */
	protected function get_partial_path($name, $paths = array())
	{
		if (Core_Strings::starts_with($name, '!')) {
			$trace = debug_backtrace();
			foreach ($trace as $k => $t)
				if (Core_Strings::ends_with($t['file'], $this->extension)) {
					break;
				}
			return Templates::add_extension(dirname($trace[$k]['file']) . '/' . substr($name, 1), $this->extension);
		}
		$file = Templates::add_extension($name, $this->extension);
		if (Templates::is_absolute_path($name)) {
			return $file;
		}
		$paths = array_merge($paths, $this->partial_paths(), Templates::option('templates_root'));
		foreach ($paths as $path) {
			$result = rtrim($path, '/') . '/' . $file;
			if (is_file($result)) {
				return $result;
			}
		}
		return $result;
	}

	/**
	 * Возвращает делегатор хелперов текущего шаблона
	 *
	 * @return Object_Aggregator
	 */
	protected function get_helpers()
	{
		return Core::if_null($this->helpers,
			$this->container ? $this->container->get_helpers() : Templates_HTML::helpers()
		);
	}

	/**
	 * Инклюдит указанный фаил, создавая необходимые переменные
	 *
	 * @param $__path
	 *
	 * @return Templates_HTML_Template
	 * @throws Templates_MissingTemplateException
	 */
	protected function load($__path)
	{
		extract($this->get_parms());
		if (IO_FS::exists($__path)) {
			include($__path);
		} else {
			throw new Templates_MissingTemplateException($__path);
		}
		return $this;
	}

	/**
	 * Возвращает конечный результат
	 *
	 * @param ArrayObject $content
	 *
	 * @return string
	 */
	protected function render_nested(ArrayObject $content = null)
	{
		Events::call('templates.render', $this, $content);
		if ($content) {
			$this->update_content($content);
		}
		$this->render_self();
		if (!empty($this->spawn_from)) {
			$this->spawn_from->update_content($this->content, array('content'));
		}
		$rc = $this->container ?
			$this->container->render_nested($this->content) :
			$this->filter($this->content['content']);
		Events::call('templates.render_result', $rc, $this, $content);
		return $rc;
	}

	protected function update_content($content, $ignore = array())
	{
		foreach ($content as $k => $v) {
			if (in_array($k, $ignore)) {
				continue;
			}
			if (is_string($v) && !empty($this->content[$k])) {
				$this->content[$k] .= $v;
			} else {
				$this->content[$k] = $v;
			}
		}
		return $this;
	}

	private function setup_frontend()
	{
		if (!class_exists('\Techart\Frontend\Frontend')) {
			throw new RuntimeException("Install techart/frontend-api:\ncomposer \"techart/frontend-api\": \"dev-master\"
			or visit http://gitlab.s.intranet/techart/frontend-api/wikis/install");
		}

		Core::load('Templates.HTML.Assets.Frontend');
		return Templates_HTML_Assets_Frontend::instance();
	}

	protected function self_content()
	{
		ob_start();
		$this->load($this->path);
		return ob_get_clean();
	}

	protected function render_self()
	{
		if ($this->cache_self) {
			$key = $this->cache_key('content', $this->parms, 'self');
			$__content = $this->cached_call($key, array($this, 'self_content'), array(), null, true);
		} else {
			$__content = $this->self_content();
		}
		$this->content['content'] = $__content; //$this->filter($__content);
	}

	public function filter($content)
	{
		$content = $this->allow_filtering ? $this->filter_custom($content) : $content;
		$content = $this->root->filter_required($content);
		return $content;
	}

	public function include_assets($place)
	{
		return $this->include_files($place);
	}

	protected function filter_required($content)
	{
		$name = 'html_required:' . $this->root->name;
		$filter = Text_Insertions::filter($name, false);
		if (!$filter->exists('head')) {
			$filter->register(array('head' => new Core_Call($this, 'build_head_insertion')));
		}
		if (!$filter->exists('assets')) {
			$filter->register(array('assets' => new Core_Call($this, 'include_assets')));
		}
		return $filter->process($content, array('layout' => $this));
	}

	protected function filter_custom($content)
	{
		return Text_Insertions::filter()->process($content, array('layout' => $this));
	}

}

//TODO: Iterator
/**
 * @method $this title(string $value)
 * @method $this description(string $value)
 * @method $this keywords(string $value)
 */
class Templates_HTML_Metas implements Core_PropertyAccessInterface,
	Core_CallInterface,
	Core_StringifyInterface
{

	protected $title = '';
	protected $http = array();
	protected $named = array();

	public function __construct()
	{
		$this->{'msapplication-config'}('none');
		Events::call('templates.metas.create', $this);
	}

	/**
	 * @param string $method
	 * @param array  $args
	 *
	 * @return $this
	 */
	public function __call($method, $args)
	{
		switch ($method) {
			case 'title':
				$this->title = $args[0];
				break;
			default:
				$this->is_http_field($method) ?
					$this->http[$this->http_field_name($method)] = $args[0] :
					$this->named[$method] = $args[0];
				break;
		}
		return $this;
	}

	/**
	 * @param string $property
	 *
	 * @return string
	 */
	public function __get($property)
	{
		switch ($property) {
			case 'title':
				return $this->title;
			default:
				return $this->is_http_field($property) ?
					(isset($this->http[$property = $this->http_field_name($property)]) ? $this->http[$property] : null) :
					(isset($this->named[$property]) ? $this->named[$property] : null);
		}
	}

	/**
	 * @param string $property
	 * @param string $value
	 *
	 * @return string
	 */
	public function __set($property, $value)
	{
		switch ($property) {
			case 'title':
				return $this->title = $value;
			default:
				$this->$property($value);
				return $value;
		}
	}

	/**
	 * @param string $property
	 *
	 * @return boolean
	 */
	public function __isset($property)
	{
		return $property == 'title' ||
		($this->is_http_field($property) ?
			isset($this->http[$this->http_field_name($property)]) :
			isset($this->named[$property]));
	}

	/**
	 * @param string $property
	 *
	 * @throws Core_UndestroyablePropertyException
	 */
	public function __unset($property)
	{
		if ($property == 'title') {
			throw new Core_UndestroyablePropertyException($property);
		}

		if ($this->is_http_field($property)) {
			if (isset($this->http[$property = $this->http_field_name($property)])) {
				unset($this->http[$property]);
			} else {
				if (isset($this->named[$property])) {
					unset($this->named[$property]);
				}
			}
		}
	}

	protected function http_to_string($name, $content)
	{
		return sprintf("<meta http-equiv=\"%s\" content=\"%s\" />\n",
			htmlspecialchars(
				Core_Arrays::join_with('-',
					Core_Arrays::map('return ucfirst(strtolower($x));',
						Core_Strings::split_by('_', $name)
					)
				)
			),
			htmlspecialchars($content)
		);
	}

	protected function named_to_string($name, $content)
	{
		return sprintf("<meta name=\"%s\" content=\"%s\" />\n", htmlspecialchars($name), htmlspecialchars($content));
	}

	/**
	 * @return string
	 */
	public function as_string()
	{
		$result = '';
		if (isset($this->http['content_type'])) {
			$result = $this->http_to_string('content_type', $this->http['content_type']);
			unset($this->http['content_type']);
		}

		$result .= sprintf("<title>%s</title>\n", htmlspecialchars($this->title));

		foreach ($this->http as $name => $content) {
			if (!empty($content)) {
				$result .= $this->http_to_string($name, $content);
			}
		}

		foreach ($this->named as $name => $content) {
			if (!empty($content)) {
				$result .= $this->named_to_string($name, $content);
			}
		}

		return $result;
	}

	/**
	 * @return string
	 */
	public function __toString()
	{
		return $this->as_string();
	}

	/**
	 * @param string $name
	 *
	 * @return boolean
	 */
	protected function is_http_field($name)
	{
		return Core_Strings::starts_with($name, 'http_');
	}

	/**
	 * @param string $name
	 *
	 * @return string
	 */
	protected function http_field_name($name)
	{
		return Core_Strings::downcase(
			Core_Regexps::replace('{^http_}', '', $name)
		);
	}
}
