<?php

use \Techart\Frontend\Frontend;
use \Techart\Frontend\Environment;
use \Techart\Frontend\PathResolver;

Core::load('Templates.HTML.Assets.EnvironmentStorage');

/**
 * Class Templates_HTML_Assets_Frontend
 *
 * @method css_url($entryPoint)
 * @method css_tag($entryPoint, $attrs = array())
 * @method js_tag($entryPoint, $attrs = array())
 * @method js_url($entryPoint)
 * @method url($path)
 * @method render($name, $args = array(), $mode = 'default')
 * @method render_block($name, $args = array(), $mode = 'default')
 * @method is_hot()
 * @method is_dev()
 * @method is_prod()
 */
class Templates_HTML_Assets_Frontend implements Core_ModuleInterface
{

	protected $env;
	private $pathResolver;
	private $manager;
	private $config;
	private static $instances = array();

	public static function instance($name = 'default')
	{
		return self::$instances[$name] ?: self::$instances[$name] = new self();
	}

	private function __construct()
	{
		$this->config = Config::all();
		$this->env = new Environment(new Templates_HTML_Assets_EnvironmentStorage($this->config, WS::env()->request));
		$this->pathResolver = new PathResolver($this->frontend_path(),$this->path_resolver_config());
		$this->manager = new Frontend($this->env, $this->pathResolver);
	}

	public function closure($callback, $params = array())
	{
		return $this->manager->closure($callback, $params);
	}

	public function get_manager()
	{
		return $this->manager;
	}

	public function __call($name, $arguments)
	{
		$name = Core_Strings::to_camel_case($name, true);
		switch($name)
		{
			case 'render':
			case 'renderBlock':
				$obj = $this->manager->templates();
				break;

			case 'url':
			case 'cssUrl':
			case 'jsUrl':
			case 'cssTag':
			case 'jsTag':
				$obj = $this->manager->assets();
				break;

			case 'isHot':
			case 'isDev':
			case 'isProd':
				$obj = $this->env;
				break;
			default:
				throw new Exception($name);
		}

		return call_user_func_array(array($obj, $name), $arguments);
	}

	private function frontend_path()
	{
		$path = '../frontend';
		if (property_exists($this->config, 'frontend') && property_exists($this->config->frontend, 'path')) {
			$path = $this->config->frontend->path;
		} elseif (property_exists($this->config, 'frontend_path')) {
			$path = $this->config->frontend_path;
		}
		return $path;
	}

	private function twig_cache_path()
	{
		$path = '../cache/twig';
		if (property_exists($this->config, 'twig_cache_path')) {
			$path = $this->config->twig_cache_path;
		}
		return $path;
	}

	private function doc_root()
	{
		$doc_root = '';
		if (property_exists($this->config, 'doc_root')) {
			$doc_root = $this->config->doc_root;
		}
		return $doc_root;
	}

	/**
	 * Метод формирует массив настроек для PathResolver. Настройки по умолчанию
	 * сливаются с настройками из конфига (frontend->path_resolver).
	 *
	 * @return array
	 */
	protected function path_resolver_config()
	{
		$config = array(
			'twigCachePath' => $this->twig_cache_path(),
			'docRoot' => $this->doc_root()
		);
		if ($this->config->frontend->path_resolver) {
			$config = array_replace_recursive($config, (array)$this->config->frontend->path_resolver);
		}
		return $config;
	}
}
