<?php
/**
 * Templates.HTML.Maps
 *
 * @package Templates\HTML\Helpers\Maps
 * @version 1.0.0
 */
Core::load('Templates.HTML');

/**
 * @package Templates\HTML\Helpers\Maps
 */
class Templates_HTML_Helpers_Maps implements Core_ModuleInterface, Templates_HelperInterface
{

	const VERSION = '1.0.0';

	/**
	 */
	static public function initialize()
	{
		Templates_HTML::use_helper('maps', 'Templates.HTML.Helpers.Maps');
	}

	/**
	 * @param Templates_HTML_Template $template
	 * @param string $service
	 * @param string $map_id
	 * @param string $name
	 * @param array $options
	 *
	 * @param string $path_to_template
	 * @return string
	 * @internal param string $id
	 */
	public function map($template, $service, $map_id, $name, $options = array(), $path_to_template = 'helpers/maps')
	{
		if (!is_array($options)) {
			$options = (array)$options;
		}

		$json_string = json_encode($options);

		$lang = 'ru';
		if($service == 'yandex') {
			$lang = 'ru_RU';
		}

		if ($options['properties']['lang'] != null) {
			$lang = $options['properties']['lang'];
		}

		if ($options['properties']['key'] != null) {
			$key = $options['properties']['key'];
		} elseif ('yandex' == $service) {
			$key = CMS::vars()->get('ymaps_key');
		}


		if ($service == null) {
			$service = 'google';
		}

		switch ($service) {
			case 'google':
				$template->use_script('https://maps.google.com/maps/api/js?sensor=false&language=' . $lang, array('type' => 'lib', 'weight' => -21));
				$template->use_script('infobubble-compiled.js', array('type' => 'lib', 'weight' => -20));
				break;
			case 'yandex':
				$template->use_script('https://api-maps.yandex.ru/2.1/?lang=' . $lang. ($key ? '&apikey='. $key : ''), array('type' => 'lib', 'weight' => -21));
				break;
			default:
				return 0;
		}
		$template->use_script('maps.js', array('type' => 'lib', 'weight' => -19));
		return $template->partial($path_to_template, array(
				'json_string' => $json_string,
				'map' => $service . '_maps',
				'name' => $name,
				'map_id' => $map_id)
		);
	}

}

