<?php
/**
 * @package Tree\DB
 */

// Base orm classes and interfaces

Core::load('DB.ORM');

class Tree_DB implements Core_ModuleInterface
{
	const VERSION = '0.0.0';

}

abstract class Tree_DB_AbstractMapper extends DB_ORM_SQLMapper
{

	abstract public function generate();

	abstract protected function map_children($item);

	abstract protected function map_siblings($item, $include_self = false);

	abstract protected function map_descendants($item);

	abstract protected function map_parent($item);

	abstract protected function map_full_tree();
}

interface Tree_DB_EntityInterface
{

	public function get_children();

	public function get_siblings($include_self = false);

	public function get_descendants();

	public function get_trail($include_self = true);

	public function get_trail_ids($include_self = true);

	public function get_parent();
}

trait Tree_DB_EntityTrait
{
	public function get_children()
	{
		return $this->mapper->children($this)->select();
	}

	public function get_siblings($include_self = false)
	{
		return $this->mapper->siblings($this, $include_self)->select();
	}

	public function get_descendants()
	{
		return $this->mapper->descendants($this)->select();
	}

	public function get_parent()
	{
		return $this->mapper->parent($this)->select_first();
	}

	public function get_root()
	{
		$trail = $this->get_trail();
		return count($trail) ? reset($trail) : array();
	}

	public function get_trail_ids($include_self = false)
	{
		$ids = array();
		foreach ($this->get_trail($include_self) as $t)
			$ids[] = $t->id();
		return $ids;
	}

	public function get_descendants_ids($include_self = false)
	{
		$ids = $include_self ? array($this->id) : array();
		foreach ($this->mapper->descendants($this)->only('id')->as_array() as $t)
			$ids[] = $t['id'];
		return $ids;
	}
}

abstract class Tree_DB_AbstractEntity extends DB_ORM_Entity implements Tree_DB_EntityInterface
{

	public function get_children()
	{
		return $this->get_mapper()->children($this)->select();
	}

	public function get_siblings($include_self = false)
	{
		return $this->get_mapper()->siblings($this, $include_self)->select();
	}

	public function get_descendants()
	{
		return $this->get_mapper()->descendants($this)->select();
	}

	public function get_parent()
	{
		return $this->get_mapper()->parent($this)->select_first();
	}

}