<?php
/**
 * WS.Session
 *
 * @package WS\Middleware\Session
 * @version 0.2.0
 */
Core::load('Net.HTTP.Session', 'WS');

/**
 * @package WS\Middleware\Session
 */
class WS_Middleware_Session implements Core_ModuleInterface
{
	const VERSION = '0.2.1';

	/**
	 * @param WS_ServiceInterface $application
	 *
	 * @return WS_ServiceInterface
	 */
	public static function Service(WS_ServiceInterface $application)
	{
		return new WS_Middleware_Session_Service($application);
	}
}

/**
 * @package WS\Middleware\Session
 */
class WS_Middleware_Session_Service extends WS_MiddlewareService
{
	protected $env;

	/**
	 * @param WS_Environment $env
	 *
	 * @return mixed
	 *
	 * @throws Exception
	 */
	public function run(WS_Environment $env)
	{
		$this->env = $env;
		$error = null;
		if (Core::is_cli() && !$env->request->session()) {
			return $this->application->run($env);
		}
		$session = $env->request->session();
		$env->flash = Net_HTTP_Session::Flash($session);

		try {
			$result = $this->application->run($env);
		} catch (Exception $e) {
			$error = $e;
		}

		Events::add_once('ws.response', array($this, 'rewrite_flash'));

		if ($error) {
			throw $error;
		} else {
			return $result;
		}
	}

	public function rewrite_flash()
	{
		$value = $this->env->flash->later;
		if ($value || $this->env->flash->is_init()) {
			$this->env->request->session()->set('flash', $value);
		}
	}
}
